/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Boucle de jeu : dessin et gestion des donnes.
 *****************************************************************************/

#ifndef BOUCLE_JEU_H
#define BOUCLE_JEU_H
//-----------------------------------------------------------------------------
#include "../include/base.h"
//-----------------------------------------------------------------------------

// Initialise le jeu
void InitJeu ();

//-----------------------------------------------------------------------------

// Etats possibles du jeu
typedef enum
{
  jeuVER_JOUE,       // Un ver est en train de jouer : peut tout faire
  jeuVER_A_JOUE,     // Un ver a jou son arme, mais peut encore se dplacer
  jeuANIMATION,      // Animation d'une arme ou d'un objet
  jeuPAUSE_FIN_TOUR, // Pause en fin du tour
  jeuANIM_FIN_TOUR   // Animation en fin de tour (caisse qui tombe)
} etat_jeu;

//-----------------------------------------------------------------------------

class BoucleJeu
{
private:
  etat_jeu etat;
  uint pause_seconde;
  uint tps_tour;
  uint tps_fin_tour;
  bool pause_fin_tour;

public:
  BoucleJeu();

  // Dessin du jeu au complet. force : force le dessin de l'cran complet ?
  void Draw (bool force_dessin);

  // La boucle principale du jeu
  void Run();

  // Refresh du jeu en entier !
  void Refresh();

  // Dbut/fin d'un tour
  void DebutTour();
  void FinTour();

  // Lit/change l'tat
  etat_jeu LitEtat() const { return etat; }
  void ChangeEtat (etat_jeu nv_etat, bool debut_jeu=false);

private:
  void BeginLoop();
  bool EndLoop();
  void CallDraw();
};

extern BoucleJeu boucle_jeu;
//-----------------------------------------------------------------------------
#endif
