/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Configuration de Wormux : toutes les variables qui sont intressantes 
 * modifier se retrouvent ici. Les variables ont une valeur par dfaut qui
 * peut tre modifie avec le fichier de configuration.
 *****************************************************************************/

#include "game_mode.h"
//-----------------------------------------------------------------------------
#include "../tool/i18n.h"
#include "game_loop.h"
#include "game.h"
#include "../weapon/all.h"
#include "../weapon/weapons_list.h"
#include <iostream>
//-----------------------------------------------------------------------------
namespace Wormux 
{
ModeJeu mode_jeu;

//-----------------------------------------------------------------------------

ModeJeu::ModeJeu()
{
  nbr_ver_max = 6;
  nbr_equipe_max = 4;
  temps_tour = 60;
  temps_fin_tour = 2;
  allow_character_selection = BEFORE_FIRST_ACTION;
}

//-----------------------------------------------------------------------------

// Load data options from the selected game_mode
bool ModeJeu::ChargeXml(xmlpp::Element *xml)
{
  std::string txt;
  if (LitDocXml::LitString (xml, "allow_character_selection", txt)) {
    if (txt == "always") 
      allow_character_selection = ALWAYS;
    else if (txt == "never") 
      allow_character_selection = NEVER;
    else if (txt == "before_first_action") 
      allow_character_selection = BEFORE_FIRST_ACTION;
  }


  LitDocXml::LitUint (xml, "temps_tour", temps_tour);
  LitDocXml::LitUint (xml, "temps_fin_tour", temps_fin_tour);
  LitDocXml::LitUint (xml, "nbr_equipe_max", nbr_equipe_max);
  LitDocXml::LitUint (xml, "nbr_ver_max", nbr_ver_max);
  
  //=== Weapons ===
  xmlpp::Element *armes = LitDocXml::AccesBalise (xml, "weapons");
  if (armes != NULL)
  {
    std::list<Arme *> l_weapons_list = gst_armes.GetList() ;
    std::list<Arme *>::iterator 
      itw = l_weapons_list.begin(), 
      end = l_weapons_list.end();

    for (; itw != end ; ++itw) {

      std::cout << "o " 
		<< Format(_("Load weapon %s"), (*itw)-> GetName().c_str())
		<< std::endl;

      (*itw)->LoadXml(armes);
    }
  }

  // Ver
  xmlpp::Element *xmlver = LitDocXml::AccesBalise (xml, "ver");
  if (xmlver != NULL)
  {
    LitDocXml::LitUint (xmlver, "energie_init", ver.energie_init);
    LitDocXml::LitUint (xmlver, "energie_max", ver.energie_max);
    LitDocXml::LitUint (xmlver, "masse", ver.masse);
    LitDocXml::LitUint (xmlver, "force_saut", ver.force_saut);
    LitDocXml::LitInt  (xmlver, "angle_saut", ver.angle_saut);
    LitDocXml::LitUint (xmlver, "force_super_saut", ver.force_super_saut);
    LitDocXml::LitInt  (xmlver, "angle_super_saut", ver.angle_super_saut);
  }
  return true;
}

//-----------------------------------------------------------------------------

bool ModeJeu::Charge (const std::string &nomfich)
{
  try
  {
    LitDocXml doc;
    if (!doc.Charge (nomfich)) return false;
    if (!ChargeXml (doc.racine())) return false;
  }
  catch (const xmlpp::exception &e)
  {
    std::cout << "Erreur lors du chargement du mode de jeu "<<nomfich<<" :" << std::endl
	      << e.what() << std::endl;
    return false;
  }
  return true;
}

//-----------------------------------------------------------------------------

bool ModeJeu::AllowCharacterSelection() const
{
  switch (mode_jeu.allow_character_selection)
  {
  case ModeJeu::ALWAYS: break;
  case ModeJeu::BEFORE_FIRST_ACTION:
    return (boucle_jeu.LitEtat() == jeuVER_JOUE) && !jeu.choix_ver_fait;
  case ModeJeu::NEVER: 
    return false;
  }
  return true;
}

//-----------------------------------------------------------------------------
}
