/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Refresh des animations : plusieurs images dfilant selon un chronomtre.
 * Il est possible de faire une rotation horizontale de l'image.
 *****************************************************************************/

#include "animation.h"
//-----------------------------------------------------------------------------
#include "rotation.h"
#include "../game/time.h"
#include <ClanLib/core.h>
using namespace Wormux;
//-----------------------------------------------------------------------------

void Animation::Init (CL_Sprite *psurface, bool autorise_rotation,
		      animation_t type, uint vitesse)
{
  Sprite::Init (psurface, autorise_rotation);
  ChangeType (type, vitesse);
}

//-----------------------------------------------------------------------------

// Change le type d'une animation
void Animation::ChangeType (animation_t ptype, uint pvitesse)
{
  assert ((0 < pvitesse) || (ptype == animARRET));
  vitesse = pvitesse;
  type = ptype;
  finie = false;
  pause = temps.Lit()+vitesse;
  m_frame = 0;
  if (type == animARRET) m_frame = 0;
}

//-----------------------------------------------------------------------------

void Animation::Refresh()
{
  // Pas d'animation ?
  if (type == animARRET) return;

  // Il est temps de faire une mise  jour ?
  if (temps.Lit() < pause) return;
  pause = temps.Lit()+vitesse;

  // Change de frame
  FrameSuivante();
}

//-----------------------------------------------------------------------------

void Animation::FrameSuivante()
{
  if (type == animARRET) return;

  if (m_frame < LitNbrFrame()-1) {
    ChangeFrame(m_frame+1);
  } else {
    if (type == animBOUCLE)
      ChangeFrame(0);
    else
      finie = true;
  }
}

//-----------------------------------------------------------------------------

void Animation::Reset()
{
  m_frame = 0;
  pause = temps.Lit()+vitesse;
  finie = false;
}

//-----------------------------------------------------------------------------

void Animation::Stop()
{ 
  type = animARRET;
  m_frame = LitNbrFrame()-1;
}

//-----------------------------------------------------------------------------

bool Animation::EstFinie() const 
{ 
  if (type == animARRET) return true;
  return finie; 
}

//-----------------------------------------------------------------------------

void Animation::Draw(int x, int y) const
{
  CL_Sprite *img = const_cast<CL_Sprite*> (this -> m_surface);
  if (m_direction == -1)
  {
    img->set_scale(-1,1);
    x +=  GetWidth();
  }
  else
    img->set_scale(1,1);
  img->set_frame(m_frame);
  img->draw (x, y);
}

//-----------------------------------------------------------------------------
