/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Classe de gestion d'une image.
 *****************************************************************************/

#include "image.h"
//-----------------------------------------------------------------------------

Image::Image ()
{
  m_surface = NULL;
  m_frame = 0;
}

//-----------------------------------------------------------------------------

Image::~Image ()
{}

//-----------------------------------------------------------------------------

void Image::Init (CL_Sprite *surface)
{ 
  m_surface = surface; 
  m_frame = 0;
}

//-----------------------------------------------------------------------------

uint Image::LitNbrFrame() const
{ return LitSprite().get_frame_count(); }

//-----------------------------------------------------------------------------

void Image::Draw(int x, int y) const
{ const_cast<Image*> (this) -> AccesSprite().draw (x, y); }

//-----------------------------------------------------------------------------

void Image::DrawFrame (int x, int y, uint frame) const
{
  assert (frame < LitNbrFrame());
  assert (m_surface != NULL);
  CL_Sprite *img = const_cast<CL_Sprite*> (this -> m_surface);
  int old_frame = img -> get_current_frame();
  img -> set_frame (frame);
  img -> draw (x,y);
  img -> set_frame (old_frame);
}

//-----------------------------------------------------------------------------

void Image::DrawScaled (int x, int y, 
			    float echelle_x, float echelle_y) const
{
  DrawSize (x,y,
		 static_cast<uint>(GetWidth()*echelle_x),
		 static_cast<uint>(GetHeight()*echelle_y));
}

//-----------------------------------------------------------------------------

void Image::DrawSize (int x, int y, uint width, uint height) const
{
  assert (m_surface != NULL);
  CL_Sprite *img = const_cast<CL_Sprite*> (this -> m_surface);
  CL_Rect rect(x,y, x+width, y+height);
  img -> draw (rect);
}

//-----------------------------------------------------------------------------

uint Image::GetWidth() const { return LitSprite().get_width(); }
uint Image::GetHeight() const { return LitSprite().get_height(); }

//-----------------------------------------------------------------------------

const CL_Sprite& Image::LitSprite() const
{
  assert (m_surface != NULL);
  assert (*m_surface);
  return *m_surface;
}

//-----------------------------------------------------------------------------

CL_Sprite& Image::AccesSprite()
{
  assert (m_surface != NULL);
  assert (*m_surface);
  return *m_surface;
}

//-----------------------------------------------------------------------------

CL_Surface Image::LitSurface() const
{
  assert (m_surface != NULL);
  assert (*m_surface);
  return m_surface -> get_frame_surface( m_surface -> get_current_frame() );
}

//-----------------------------------------------------------------------------

void Image::FrameSuivante()
{
  if (m_frame < LitNbrFrame()-1) m_frame++; else m_frame = 0;
}

//-----------------------------------------------------------------------------

void Image::ChangeFrame(uint frame)
{
  assert (frame < LitNbrFrame());
  if(m_frame!=frame)
  {
    m_surface->set_frame(frame);
  }
  m_frame = frame;
}

//-----------------------------------------------------------------------------

uint Image::LitFrame() const { return m_frame; }

//-----------------------------------------------------------------------------
