/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Rotation d'une image.
 *****************************************************************************/

#include "rotation.h"
//-----------------------------------------------------------------------------

#ifdef PORT_CL07

// Flip horizontal (image en 8 bpp)
void FlipHorizontal_8 (CL_PixelBuffer &src, 
		       uchar *dst, uint dst_pitch,
		       uint larg, uint haut)
{
  uchar *sauve_dst=dst;
  float r,v,b,a;
  for (uint iy=0; iy<haut; ++iy)
  {
    dst = sauve_dst;
    for (uint ix=0; ix<larg; ++ix)
    {
      src.get_pixel (larg-1-ix, iy, &r,&v,&b,&a);
      *dst++ = (uchar)(a*255);
      *dst++ = (uchar)(b*255);
      *dst++ = (uchar)(v*255);
      *dst++ = (uchar)(r*255);
    }
    sauve_dst += dst_pitch;
  }
}

//-----------------------------------------------------------------------------

// Flip horizontal (image en 32bpp)
void FlipHorizontal_32 (uint8 *src, uint8 *dst,
			uint src_pitch, uint dst_pitch,
			uint larg, uint haut)
{
  src += (larg-1)*4;
  uint8 *sauve_src=src, *sauve_dst=dst;
  for (uint iy=0; iy<haut; ++iy)
  {
    src = sauve_src;
    dst = sauve_dst;
    for (uint ix=0; ix<larg; ++ix)
    {
      *(uint32 *)dst = *(uint32 *)src;
      src -= 4;
      dst += 4;
    }
    sauve_src += src_pitch;
    sauve_dst += dst_pitch;
  }
}

//-----------------------------------------------------------------------------

// Flip horizontal (image en 32bpp)
CL_Surface FlipHorizontal (const CL_Surface &surface)
{
  CL_PixelBuffer *provider = surface.get_provider();
  const uint nbr_frame = provider -> get_num_frames();
  const uint larg = provider -> get_width();
  const uint haut = provider -> get_height();
  const uint src_pitch = provider -> get_pitch();
  CL_Canvas* canvas = new CL_Canvas(larg,haut,nbr_frame);
  const uint dst_pitch = canvas -> get_pitch();
  int bpp = provider -> get_depth();

  // Il faut absolument une image en 32bpp
  assert ((bpp == 8) || (bpp == 32));

  // Accde aux donnes
  canvas -> lock();
  uint8 *dst = (uchar*)canvas -> get_data();

  switch (bpp)
  {
  case 8: 
  {
    CL_PixelBuffer *src;
    src = const_cast<CL_PixelBuffer*> (surface.get_provider());
    FlipHorizontal_8 (*src, dst, dst_pitch, 
		      larg, haut*nbr_frame); 
    break;
  }

  case 32:
  {
    surface.get_provider() -> lock();
    uint8 *src = (uchar*)provider -> get_data();
    FlipHorizontal_32 (src, dst, src_pitch, dst_pitch, larg, haut*nbr_frame); 
    provider -> unlock();
    break;
  }

  default: break;
  }

  canvas -> unlock();
  return CL_Surface (canvas, true);
}

//-----------------------------------------------------------------------------

#endif
