/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Keyboard managment. Use ClanLIB event.
 *****************************************************************************/

#include "keyboard.h"
//-----------------------------------------------------------------------------
#include <sstream>
#include "../team/macro.h"
#include "../game/time.h"

#include "../weapon/weapon.h"
#include "../weapon/weapons_list.h"

#include "game_msg.h"
#include "../map/camera.h"
#include "interface.h"
#include "../team/move.h"
#include "../tool/i18n.h"
#include "../game/game.h"
#include "../game/game_loop.h"
#include "../game/game_mode.h"
#include "../sound/jukebox.h"
#include "../game/config.h"
#include "../graphic/video.h"
#include "cursor.h"
#include "../include/constant.h"
using namespace Wormux;
//-----------------------------------------------------------------------------

// Active le mode tricheur ?
#ifdef DEBUG
#  define MODE_TRICHEUR
#endif

// Vitesse du definalement au clavier
#define SCROLL_CLAVIER 20 // ms

// Delta appliqu  l'angle du viseur
#define DELTA_VISEUR 2

#define USE_HAND_POSITION_MODIFIER

//-----------------------------------------------------------------------------
Clavier clavier;
//-----------------------------------------------------------------------------

Clavier::Clavier()
{
  pilote_installe = false;
}

//-----------------------------------------------------------------------------

void Clavier::DesinstallePilote()
{
  assert (pilote_installe);
  CL_Keyboard::sig_key_up().disconnect(slot);
  pilote_installe = false;
}

//-----------------------------------------------------------------------------

void Clavier::InstallePilote()
{
  assert (!pilote_installe);
  slot = CL_Keyboard::sig_key_up().connect(this, &Clavier::TraiteTouche);
  pilote_installe = true;
}

//-----------------------------------------------------------------------------

// Traite une touche relache
void Clavier::TraiteTouche (const CL_InputEvent &touche)
{
  // Fin du jeu ?
  if (touche.id == CL_KEY_ESCAPE)
  {
    jeu.fin_partie = true;
    return;
  }

  // P: Pause
  if (touche.id == CL_KEY_P)
  {
    jeu.Pause();
    return;
  }

#ifdef MODE_TRICHEUR
  // Page haut/bas : Change la vitesse du jeu
  if (touche.id == CL_KEY_PRIOR)
  {
    temps.ChangeVitesse (temps.LitVitesse()*2, true);
    return;
  }
  if (touche.id == CL_KEY_NEXT)
  {
    temps.ChangeVitesse (temps.LitVitesse()/2, true);
    return;
  }
#endif


#ifdef USE_HAND_POSITION_MODIFIER
  bool meta = 
    CL_Keyboard::get_keycode(CL_KEY_LSHIFT)
    || CL_Keyboard::get_keycode(CL_KEY_RSHIFT);

  // Shift: Affiche/cache le niveau d'nergie des vers
  if (meta)
  {
    int frame = ActiveCharacter().image.get_current_frame();
    skin_translate_t &tr = 
      ActiveCharacter().AccessSkin().walking.hand_position[frame];
    if (touche.id == CL_KEY_LEFT) { tr.dx--; return; }
    if (touche.id == CL_KEY_RIGHT) { tr.dx++; return; }
    if (touche.id == CL_KEY_UP) { tr.dy--; return; }
    if (touche.id == CL_KEY_DOWN) { tr.dy++; return; }
  }
#endif

  // F10: Mode plein cran on/off
  if (touche.id == CL_KEY_F10)
  {
    // Tente de passer en plein cran ...
    video.SetFullScreen( !video.IsFullScreen() );
    return;
  }

  // F9: Affichage ou non de l'interface (en bas)
  if (touche.id == CL_KEY_F9)
  {
    interface.ChangeAffiche (!interface.EstAffiche());
    return;
  }

  // Shift: Affiche/cache le niveau d'nergie des vers
  if ((touche.id == CL_KEY_LSHIFT) || (touche.id == CL_KEY_RSHIFT))
  {
    config.affiche_energie_ver = !config.affiche_energie_ver;
    return;
  }

  // On charge une arme...
  if (ActiveTeam().LitArme().IsLoading())
    return;

  // On veut changer de ver ?
  if (touche.id == CL_KEY_TAB)
  {
    if (mode_jeu.AllowCharacterSelection()) ActiveTeam().NextCharacter();
    return;
  }

  if (boucle_jeu.LitEtat() == jeuVER_JOUE)
  {
    // Choosing the weapon ?
    if (touche.id == CL_KEY_F1 && ActiveTeam().LitArme().CanChangeWeapon() )
    {
      Arme * weapon = gst_armes.GetWeaponBySort(1) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return ;
    }
    if (touche.id == CL_KEY_F2 && ActiveTeam().LitArme().CanChangeWeapon())
    {
      Arme * weapon = gst_armes.GetWeaponBySort(2) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return;
    }
    if (touche.id == CL_KEY_F3 && ActiveTeam().LitArme().CanChangeWeapon())
    {
      Arme * weapon = gst_armes.GetWeaponBySort(3) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return;
    }
    if (touche.id == CL_KEY_F4 && ActiveTeam().LitArme().CanChangeWeapon())
    {
      Arme * weapon = gst_armes.GetWeaponBySort(4) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return;
    }
    if (touche.id == CL_KEY_F5 && ActiveTeam().LitArme().CanChangeWeapon())
    {
      Arme * weapon = gst_armes.GetWeaponBySort(5) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return;
    }
    if (touche.id == CL_KEY_F6 && ActiveTeam().LitArme().CanChangeWeapon())
    {
      Arme * weapon = gst_armes.GetWeaponBySort(6) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return;
    }
    if (touche.id == CL_KEY_F7 && ActiveTeam().LitArme().CanChangeWeapon())
    {
      Arme * weapon = gst_armes.GetWeaponBySort(7) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return;
    }
    if (touche.id == CL_KEY_F8 && ActiveTeam().LitArme().CanChangeWeapon())
    {
      Arme * weapon = gst_armes.GetWeaponBySort(8) ;
      if (weapon != NULL)
	ActiveTeam().ChangeArme(weapon);
      return;
    }

    // ESPACE: Tire avec l'arme
    if (
	(touche.id == CL_KEY_SPACE) 
	&& (boucle_jeu.LitEtat() == jeuVER_JOUE)
	&& ( ! ActiveTeam().LitArme().IsLoading() )
	&& ( ActiveTeam().LitArme().IsReady() )
	)
    {
      if (ActiveTeam().AccesArme().Shoot ()) return;
    }
  }

  // C: Demande d'autocadrage ?
  if (touche.id == CL_KEY_C)
  {
    curseur_ver.SuitVerActif();
    camera.ChangeObjSuivi (&ActiveCharacter(), true, true, true);
    return;
  }

  // M: Affiche/cache le menu des armes
  if (touche.id == CL_KEY_M)
  {     
    interface.menu_armes.ChangeAffichage();
    return;
  }
}

//-----------------------------------------------------------------------------

void Clavier::TestCamera()
{
  if (CL_Keyboard::get_keycode(CL_KEY_NUMPAD4)) {
    camera.SetXY (-SCROLL_CLAVIER, 0);
    camera.autorecadre = false;
    return;
  }
    
  if (CL_Keyboard::get_keycode(CL_KEY_NUMPAD6)) {
    camera.SetXY (SCROLL_CLAVIER, 0);
    camera.autorecadre = false;
    return;
  }

  if (CL_Keyboard::get_keycode(CL_KEY_NUMPAD8)) {
    camera.SetXY (0, -SCROLL_CLAVIER);
    camera.autorecadre = false;
    return;
  }

  if (CL_Keyboard::get_keycode(CL_KEY_NUMPAD2)) {
    camera.SetXY (0,SCROLL_CLAVIER);
    camera.autorecadre = false;
    return;
  }
}

//-----------------------------------------------------------------------------

// Refresh des touches du clavier
void Clavier::Refresh()
{
  if ( ActiveTeam().LitArme().IsLoading() )
  {

    // Relax Space key ... or strength == max strength !
    if (!CL_Keyboard::get_keycode(CL_KEY_SPACE) 
	|| ActiveTeam().LitArme().ReadStrength() == ActiveTeam().LitArme().max_strength)
      {
	ActiveTeam().AccesArme().StopLoading();
	if(ActiveTeam().AccesArme().Shoot())
        {
          boucle_jeu.ChangeEtat (jeuVER_A_JOUE);
          return;
        }
      }
    
    // still pressing the Space key 
    ActiveTeam().AccesArme().UpdateStrength();
    
  } else { // Begin to load
    if (CL_Keyboard::get_keycode(CL_KEY_SPACE)
	&& (boucle_jeu.LitEtat() == jeuVER_JOUE)
	&& ActiveTeam().LitArme().IsReady())
      {
	ActiveTeam().AccesArme().InitLoading();
      }
  }

  // Le ver est pret a faire une action ?
  if (ActiveCharacter().IsReady() && !ActiveCharacter().IsDead())
  {
    // On veut sauter ?
    if (CL_Keyboard::get_keycode(CL_KEY_ENTER)) 
    { 
      jeu.choix_ver_fait = true;
      ActiveCharacter().Saute(); 
      return; 
    }
    if (CL_Keyboard::get_keycode(CL_KEY_BACKSPACE)) 
    { 
      jeu.choix_ver_fait = true;
      ActiveCharacter().SuperSaut(); 
      return; 
    }
  
    // Move the crosshair if the selected weapon agree (method ChangeAngle verify it)
    if (ActiveTeam().viseur.actif)
    {
      if (CL_Keyboard::get_keycode(CL_KEY_UP)) {
	ActiveTeam().viseur.ChangeAngle (-DELTA_VISEUR);
      } else if (CL_Keyboard::get_keycode(CL_KEY_DOWN)) {
	ActiveTeam().viseur.ChangeAngle (DELTA_VISEUR);
      }
    }

#ifdef USE_HAND_POSITION_MODIFIER
    bool meta = 
      CL_Keyboard::get_keycode(CL_KEY_LSHIFT)
      || CL_Keyboard::get_keycode(CL_KEY_RSHIFT);
    if (meta) return;
#endif

    // Va vers la droite ?
    if (CL_Keyboard::get_keycode(CL_KEY_RIGHT)) {
      MoveCharacterRight (ActiveCharacter());

    // Va vers la gauche ?
    } else if (CL_Keyboard::get_keycode(CL_KEY_LEFT)) {
      MoveCharacterLeft (ActiveCharacter());
    }
  }
}

//-----------------------------------------------------------------------------
