/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Refresh de la souris.
 *****************************************************************************/

#include "mouse.h"
//-----------------------------------------------------------------------------
#include "../team/macro.h"
#include "../game/game_mode.h"
#include "../map/camera.h"
#include "../game/game.h"
#include "../game/game_loop.h"
#include "../game/time.h"
#include "../tool/geometry_tools.h"
#include "../include/constant.h" // FOND_X, FOND_Y
#include "interface.h"
#include "cursor.h"
#include "../weapon/weapon.h"
using namespace Wormux;
//-----------------------------------------------------------------------------
Souris souris;
//-----------------------------------------------------------------------------

// Vitesse du definalement  la souris
const uint SCROLL_SOURIS = 20;

// Largeur de la zone de sensibilite au camera a la souris
const uint SENSIB_SCROLL_SOURIS = 50; // pixels

// Active le mode tricheur ?
#ifdef DEBUG
#  define MODE_TRICHEUR
#endif

//-----------------------------------------------------------------------------

Souris::Souris()
{
  scroll_actif = false;
}


//-----------------------------------------------------------------------------

void Souris::Reset()
{}

//-----------------------------------------------------------------------------

#ifdef PORT_CL07

bool Souris::ClicG() const { return CL_Mouse::left_pressed(); }
bool Souris::ClicD() const { return CL_Mouse::right_pressed(); }
bool Souris::ClicM() const { return CL_Mouse::middle_pressed(); }

#else

bool Souris::ClicG() const { return false; }
bool Souris::ClicD() const { return false; }
bool Souris::ClicM() const { return false; }

#endif

//-----------------------------------------------------------------------------

bool Souris::ActionClicD()
{ 
  if ( ActiveTeam().LitArme().CanChangeWeapon() )
  {
    interface.menu_armes.ChangeAffichage();
  }
  return true;
}

//-----------------------------------------------------------------------------

bool Souris::ActionClicG()
{
  const CL_Point pos_monde = GetPosMonde();

#ifdef MODE_TRICHEUR
  // Control gauche + clic gauche = tlportation !
  if (CL_Keyboard::get_keycode(CL_KEY_CONTROL) && ActiveCharacter().IsReady())
  {
    ActiveCharacter().SetXY (pos_monde.x, pos_monde.y);
    ActiveCharacter().PhysicalEngine();
    return true;
  }
#endif
	
  // Action dans le menu des armes ?
  if (interface.menu_armes.ActionClic (GetX(),GetY())) 
  {
    return true;
  } 


  // On peut changer de ver ?
  if (mode_jeu.AllowCharacterSelection())
  {
    // Slection d'un ver se son quipe ?
    bool ver_choisi=false;
    Team::iterator 
      it=ActiveTeam().Debut(),
      fin=ActiveTeam().Fin();
    uint index=0;
    for (; it != fin; ++it, ++index)
    {
      if (&(*it) != &ActiveCharacter()
	  && !it -> IsDead() 
	  && DansRect (it -> GetRect(), pos_monde))
      {
	ver_choisi = true;
	break;
      }
    }
    if (ver_choisi)
    {
      ActiveTeam().SelectCharacter (it);
      return true;
    }

    if (DansRect (ActiveCharacter().GetRect(), pos_monde))
    {
      curseur_ver.SuitVerActif();
      return true;
    }
  }

  // Action dans le menu des armes ?
  if (interface.menu_armes.ActionClic (GetX(),GetY())) 
  {
    return true;
  }

  // Choosing target for a weapon, many posibilities :
  // - Do nothing
  // - Choose a target but don't fire
  // - Choose a target and fire it !
  if (boucle_jeu.LitEtat() == jeuVER_JOUE) {
    ActiveTeam().AccesArme().ChooseTarget();
    return true ;
  }
  
  return false;
}

//-----------------------------------------------------------------------------

void Souris::ChoixVerPointe()
{
  if (boucle_jeu.LitEtat() != jeuVER_JOUE) return;

  const CL_Point pos_monde = GetPosMonde();

  // Quel ver est point par la souris ? (en dehors du ver actif)
  interface.ver_pointe_souris = NULL;
  POUR_TOUS_VERS_VIVANTS(equipe,ver)
  {
    if ((&(*ver) != &ActiveCharacter())
	&& DansRect (ver -> GetRect(), pos_monde))
    {
      interface.ver_pointe_souris = &(*ver);
    }
  }

  // Aucun ver n'est point ... et le ver actif alors ?
  if ((interface.ver_pointe_souris == NULL)
      && DansRect (ActiveCharacter().GetRect(), pos_monde))
  {
    interface.ver_pointe_souris = &ActiveCharacter();
  }

  // Dessine le curseur autour du ver point s'il y en a un
  if (interface.ver_pointe_souris != NULL) {
    curseur_ver.PointeObj (interface.ver_pointe_souris);
  } else {
    curseur_ver.PointeAucunObj();
  }
}

//-----------------------------------------------------------------------------
 
void Souris::TestCamera()
{
 const bool demande_scroll = CL_Keyboard::get_keycode(CL_KEY_CONTROL);

  if (demande_scroll)
  {
    if (scroll_actif) {
      int dx = sauve_x - CL_Mouse::get_x();
      int dy = sauve_y - CL_Mouse::get_y();
      camera.SetXY (dx, dy);
      camera.autorecadre = false;
    } else {
      scroll_actif = true;
    }
    sauve_x = CL_Mouse::get_x();
    sauve_y = CL_Mouse::get_y();
  } else {
    scroll_actif = false;
  }
}

//-----------------------------------------------------------------------------

void Souris::Refresh()
{
  if (!scroll_actif) ChoixVerPointe();
}

//-----------------------------------------------------------------------------

int Souris::GetX() const { return CL_Mouse::get_x(); }
int Souris::GetY() const { return CL_Mouse::get_y(); }
int Souris::GetXmonde() const { return GetX() -FOND_X +camera.GetX(); }
int Souris::GetYmonde() const { return GetY() -FOND_Y +camera.GetY(); }
CL_Point Souris::GetPosMonde() const
{ return CL_Point (GetXmonde(), GetYmonde()); }

//-----------------------------------------------------------------------------

void Souris::DesinstallePilote()
{
  assert (pilote_installe);
  CL_Mouse::sig_key_up().disconnect(slot);
  pilote_installe = false;
}

//-----------------------------------------------------------------------------

void Souris::InstallePilote()
{
  assert (!pilote_installe);
  slot = CL_Mouse::sig_key_up().connect(this, &Souris::TraiteClic);
  pilote_installe = true;
}

//-----------------------------------------------------------------------------

// Traite une touche relache
void Souris::TraiteClic (const CL_InputEvent &event)
{
  // Clic gauche de la souris ?
  if (event.id == CL_MOUSE_RIGHT)
  {
    ActionClicD();
    return;
  }

  // Clic gauche de la souris ?
  if (event.id == CL_MOUSE_LEFT)
  {
    ActionClicG();
    return;
  }
}

//-----------------------------------------------------------------------------
