/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Interface affichant diffrentes informations sur la jeu.
 *****************************************************************************/

#include "weapon_menu.h"
//-----------------------------------------------------------------------------
#include <sstream>
#include <math.h>
#include "interface.h"
#include "../map/camera.h"
#include "../team/teams_list.h"
#include "../tool/string_tools.h"
#include "../weapon/weapons_list.h"
#include "../graphic/graphism.h"
#include "../graphic/video.h"
#include "../game/time.h"
#include "../team/team.h"
#include "../interface/mouse.h"

using namespace Wormux;
//-----------------------------------------------------------------------------

//Menu des armes
const uint LARG_ICO_ARME = 24;
const uint HAUT_ICO_ARME = 24;
const uint BORD_ICO_ARME = 1;
const uint DST_ICO_ARME = 8;
const uint TPS_ICO_AFFICHAGE = 500; //temps pour afficher tous les icones (en ms)

const uint TPS_ICO_ZOOM = 150; //temps pour pour zoomer un icone

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
ItemMenuArme::ItemMenuArme(uint num_sort)
{
  weapon_type = num_sort;
  Reset();
}

//-----------------------------------------------------------------------------
void ItemMenuArme::Reset()
{
  scale = 0;
  zoom = false;
  dezoom = false;
}

//-----------------------------------------------------------------------------
void ItemMenuArme::InitZoom()
{
  if(dezoom)
    tps_debut_zoom = temps.Lit() - (TPS_ICO_ZOOM - (temps.Lit()-tps_debut_zoom));
  else
    tps_debut_zoom = temps.Lit();

  zoom = true;
  dezoom = false;
}

//-----------------------------------------------------------------------------
void ItemMenuArme::InitDezoom()
{
  if(zoom)
    tps_debut_zoom = temps.Lit() - (TPS_ICO_ZOOM - (temps.Lit()-tps_debut_zoom));
  else
    tps_debut_zoom = temps.Lit();

  dezoom = true;
  zoom = false;
}

//-----------------------------------------------------------------------------
void ItemMenuArme::ChangeZoom()
{
  if(!zoom && scale < (int)(DST_ICO_ARME - BORD_ICO_ARME))
    InitZoom();
  else
    InitDezoom();
}

//-----------------------------------------------------------------------------
void ItemMenuArme::CalculeScale()
{
  if(zoom)
  {
    scale = (int)((sin(((double)temps.Lit() - tps_debut_zoom) / TPS_ICO_ZOOM) * M_PI/2)
            * (DST_ICO_ARME - BORD_ICO_ARME));
    if(scale >= (int)(DST_ICO_ARME - BORD_ICO_ARME))
    {
      zoom = false;
      scale = (DST_ICO_ARME - BORD_ICO_ARME);
    }
  }
  else
  if(dezoom)
  {
    scale = DST_ICO_ARME - BORD_ICO_ARME;
    scale -= (int)((sin(((double)temps.Lit() - tps_debut_zoom) / TPS_ICO_ZOOM) * M_PI/2)
            * (DST_ICO_ARME - BORD_ICO_ARME));
    if(scale <= (int)0)
    {
      dezoom = false;
      scale = (int)0;
    }
  }
}

//-----------------------------------------------------------------------------
bool ItemMenuArme::SourisDessus(int s_x, int s_y)
{
  CL_Rect rect;

  rect.left = interface.menu_armes.GetX() + x - scale;
  rect.top = interface.menu_armes.GetY() + y - scale;
  rect.right = rect.left + LARG_ICO_ARME + 2*scale;
  rect.bottom = rect.top + HAUT_ICO_ARME + 2*scale;

  CL_Point point;
  point.x = s_x;
  point.y= s_y;

  if (rect.is_inside(point))
    return true;
  else
  {
    if(scale > 0 && !dezoom)
      dezoom = true;
    return false;
  }
}
//-----------------------------------------------------------------------------
//Affiche un bouton
void ItemMenuArme::Draw()
{
  CalculeScale();
  int n_x = interface.menu_armes.GetX() + x - scale;
  int n_y = interface.menu_armes.GetY() + y - scale;

  CL_Rect rect;
  rect.left = n_x;
  rect.top = n_y;
  rect.right = (n_x+LARG_ICO_ARME+BORD_ICO_ARME*2)+2*scale ;
  rect.bottom = (n_y+HAUT_ICO_ARME+BORD_ICO_ARME*2)+2*scale ;
  
  CL_Display::draw_rect (rect,CL_Color::black);
  
  rect.left += BORD_ICO_ARME;
  rect.top += BORD_ICO_ARME;
  rect.right = rect.left+LARG_ICO_ARME+2*scale;
  rect.bottom = rect.top+HAUT_ICO_ARME+2*scale;
  
  
  CL_Display::fill_rect (rect, CL_Color(45,148,139)) ;

  CL_Surface icon = arme->icone;

  icon.set_scale(double(LARG_ICO_ARME+2*scale)/double(icon.get_width()),
		 double(HAUT_ICO_ARME+2*scale)/double(icon.get_height())
		 );
  icon.draw(n_x + BORD_ICO_ARME, n_y + BORD_ICO_ARME);
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
MenuArmes::MenuArmes()
{
  affiche = false;
  montre = false;
  cache = false;
  nbr_weapon_type = 0;
}

//-----------------------------------------------------------------------------
//Ajoute une nouvelle arme au menu
void MenuArmes::NewItem(Arme * new_item, uint num_sort)
{
  ItemMenuArme item(num_sort);
  item.x = 0;
  item.y = 0;
  item.arme = new_item;
  boutons.push_back (item);

  if(num_sort>nbr_weapon_type)
    nbr_weapon_type = num_sort;
}
  
//-----------------------------------------------------------------------------
//Affichage du menu (initialisation de l'animation)
void MenuArmes::Montre()
{
  if(cache)
    tps_debut_mvt = temps.Lit() - (TPS_ICO_AFFICHAGE - (temps.Lit()-tps_debut_mvt));
  else
    tps_debut_mvt = temps.Lit();

  affiche = true;
  montre = true;
  cache = false;
}

//-----------------------------------------------------------------------------
void MenuArmes::CalculateSize()
{
  //Calcule du nombre d'icone max en hauteur:
  max_weapon = 0;
  uint nbr_current_type = 0;

  iterator it=boutons.begin(), fin=boutons.end();
  for (; it != fin; ++it)
  {
    if(it != boutons.begin())
    if(((it-1)->weapon_type) != (it->weapon_type))
    {
      if(nbr_current_type > max_weapon)
        max_weapon = nbr_current_type;
      nbr_current_type = 0;
    }
    if(ActiveTeam().ReadNbBullets(it->arme->GetName())>0
    || ActiveTeam().ReadNbBullets(it->arme->GetName())==MUNITION_ILLIMITEES)
      nbr_current_type++;
  }

  if(nbr_current_type > max_weapon)
    max_weapon = nbr_current_type;
}

//-----------------------------------------------------------------------------
void MenuArmes::Cache()
{
  if(montre)
    tps_debut_mvt = temps.Lit() - (TPS_ICO_AFFICHAGE - (temps.Lit()-tps_debut_mvt));
  else
    tps_debut_mvt = temps.Lit();

  cache = true;
  montre = false;
}

//-----------------------------------------------------------------------------
void MenuArmes::ChangeAffichage()
{
  if(affiche && !cache)
    Cache();
  else
    Montre();
}

//-----------------------------------------------------------------------------

int MenuArmes::GetX() const
{ return video.GetWidth()-GetWidth();}

//-----------------------------------------------------------------------------

int MenuArmes::GetY() const 
{
  int y = video.GetHeight();
  if (interface.EstAffiche()) y -= interface.GetHeight();
  return y;
}

//-----------------------------------------------------------------------------

int MenuArmes::GetWidth() const
{
  return DST_ICO_ARME + ((nbr_weapon_type +1)
	 * (LARG_ICO_ARME + DST_ICO_ARME));
}

//-----------------------------------------------------------------------------

int MenuArmes::GetHeight() const
{
  return -((DST_ICO_ARME + ((DST_ICO_ARME + LARG_ICO_ARME) *
          max_weapon)));
}

//-----------------------------------------------------------------------------

void MenuArmes::Init()
{
//  CL_ResourceManager* res=graphisme.LitRes();
//  image.Init (CL_Surface::load("interface/menu_arme", res));
}

//-----------------------------------------------------------------------------

void MenuArmes::MouvementMontre(int nbr_bouton,int numero_bouton,iterator it,int colonne)
{
  int delta_t=TPS_ICO_AFFICHAGE/(2*nbr_bouton);
  if((temps.Lit() > tps_debut_mvt + (delta_t*numero_bouton))
  && (temps.Lit() < tps_debut_mvt + (TPS_ICO_AFFICHAGE/2)+(delta_t*(numero_bouton))))
  {
    double delta_sin = -(asin((colonne+1.0)/(colonne+2.0)) - (M_PI/2));

    uint tps = temps.Lit() - (tps_debut_mvt + delta_t*numero_bouton);

    double tps_sin = ((double)tps * ((M_PI/2) + delta_sin)/(TPS_ICO_AFFICHAGE/2));

    it-> x -= (int)(sin(tps_sin) * (LARG_ICO_ARME + DST_ICO_ARME) * (colonne+2.0));
    it-> x += ((LARG_ICO_ARME + DST_ICO_ARME) * (colonne+1));
  }
  else
  if(temps.Lit() < tps_debut_mvt + (delta_t*numero_bouton))
  {
    it-> x += ((LARG_ICO_ARME + DST_ICO_ARME) * (colonne+1));
  }

  if(temps.Lit() > tps_debut_mvt + TPS_ICO_AFFICHAGE)
  {
    montre = false;
  }
}

//-----------------------------------------------------------------------------

bool MenuArmes::MouvementCache(int nbr_bouton,int numero_bouton,iterator it,int colonne)
{
  int delta_t=TPS_ICO_AFFICHAGE/(2*nbr_bouton);
  if((temps.Lit() > tps_debut_mvt + (delta_t*(nbr_bouton-numero_bouton)))
  && (temps.Lit() < tps_debut_mvt + (TPS_ICO_AFFICHAGE/2)+(delta_t*(nbr_bouton-numero_bouton))))
  {
    double delta_sin = -(asin((colonne+1.0)/(colonne+2.0)) - (M_PI/2));

    uint tps = temps.Lit() - (tps_debut_mvt + delta_t*(nbr_bouton-numero_bouton));
    double tps_sin = ((double)tps * ((M_PI/2) + delta_sin)/(TPS_ICO_AFFICHAGE/2));
    tps_sin = ((M_PI/2) + delta_sin) - tps_sin;

    it-> x -= (int)(sin(tps_sin) * (LARG_ICO_ARME + DST_ICO_ARME) * (colonne+2.0));
    it-> x += (LARG_ICO_ARME + DST_ICO_ARME) * (colonne+1);
  }
  else
  if(temps.Lit() > tps_debut_mvt + (delta_t*(nbr_bouton-numero_bouton)))
  {
    it-> x += (LARG_ICO_ARME + DST_ICO_ARME) * (colonne+1);
    it-> Reset();
  }

  if(temps.Lit() > tps_debut_mvt + TPS_ICO_AFFICHAGE)
  {
    cache = false;
    affiche = false;
    return true;
  }
  return false;
}

//-----------------------------------------------------------------------------

void MenuArmes::Draw()
{
  if (!affiche) return;

  int mouse_x = souris.GetPosMonde().x - camera.GetX();
  int mouse_y = souris.GetPosMonde().y - camera.GetY();

  ReactionSouris(mouse_x,mouse_y);
  CalculateSize();

  uint nbr_bouton = max_weapon * nbr_weapon_type;
  uint numero_bouton = 0;
  uint current_type = 0;

  iterator it=boutons.begin(), fin=boutons.end();
  for (it=boutons.begin(); it != fin; ++it)
  {
    if(ActiveTeam().ReadNbBullets(it->arme->GetName())<=0
    && ActiveTeam().ReadNbBullets(it->arme->GetName())!=MUNITION_ILLIMITEES)
      continue;

    if(it->weapon_type!=current_type)
    {
      numero_bouton = 0;
      current_type = it->weapon_type;
    }

    int colonne = nbr_weapon_type - current_type;
    int ligne = numero_bouton;

    it->x = GetWidth() - ((LARG_ICO_ARME + DST_ICO_ARME) * (colonne+1));
    it->y = GetHeight() + DST_ICO_ARME + (ligne * (DST_ICO_ARME+HAUT_ICO_ARME));

    if(montre)
    {
      MouvementMontre(nbr_bouton,(colonne * max_weapon) + ligne,it,colonne);
    }
    else
    if(cache)
    {
      if(MouvementCache(nbr_bouton,(colonne * max_weapon) + ligne,it,colonne))
        return;
    }

    it->Draw();
    numero_bouton++;
  }
}

//-----------------------------------------------------------------------------
void MenuArmes::ReactionSouris (int x, int y)
{
  static int bouton_sous_souris = -1; //bouton survol par la souris
  CL_Rect rect;

  rect.left = GetX();
  rect.top = GetY() + GetHeight();
  rect.right = GetX() + GetWidth();
  rect.bottom = GetY();

  CL_Point point;
  point.x = x;
  point.y = y;

  if (rect.is_inside(point))
  {
    //Bouton en cour d'analyse:
    int numero_bouton=0;
    int nv_bouton_sous_souris=-1;
    iterator it=boutons.begin(), fin=boutons.end();
    interface.arme_pointe_souris = NULL;
    for (; it != fin; ++it)
    {
      if(it->SourisDessus(x,y))
      {
          interface.arme_pointe_souris = it->arme;
        nv_bouton_sous_souris = numero_bouton;
        if(numero_bouton != bouton_sous_souris)
        {//Le curseur arrive sur un nouveau bouton
          it->ChangeZoom();
        }
      }
      else
      if(numero_bouton == bouton_sous_souris)
      {
        it->ChangeZoom();
      }
      numero_bouton++;
    }
    bouton_sous_souris = nv_bouton_sous_souris;
  }
  else
  {
    iterator it=boutons.begin();
    if(bouton_sous_souris != -1)
    {
      (it+bouton_sous_souris)->ChangeZoom();
    }
    bouton_sous_souris = -1;
    interface.arme_pointe_souris = NULL;
  }
}

//-----------------------------------------------------------------------------

bool MenuArmes::ActionClic (int x, int y)
{
  CL_Rect rect;
  if (!affiche) return false;

  iterator it=boutons.begin(), fin=boutons.end();
  for (; it != fin; ++it)
  {
    if (it->SourisDessus(x,y))
    {

      ActiveTeam().ChangeArme (it -> arme);
      ChangeAffichage();
      return true;
    }
  }
  return false;
}

//-----------------------------------------------------------------------------
