/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Refresh de l'eau pouvant apparaitre en bas du terrain.
 *****************************************************************************/

#include "water.h"
//-----------------------------------------------------------------------------
#include "../game/time.h"
#include "map.h"
#include "../graphic/graphism.h"
using namespace Wormux;

Eau eau;
//-----------------------------------------------------------------------------

// Vitesse d'animation des vagues
#define TEMPS_EAU 10

// le pas d'avance des vagues en pixels
#define AVANCE_VAGUES 1
const uint HAUTEUR_VAGUE = 5;
const uint HAUTEUR = 100;
const uint TEMPS_AVANT_MONTEE = 1; // min
const uint PAS_DE_LA_MONTEE = 1; // min
//-----------------------------------------------------------------------------

void Eau::Init()
{ 
  surface = CL_Surface("gfx/eau1", graphisme.LitRes());
  fond = CL_Surface("gfx/fond_eau", graphisme.LitRes());
}

//-----------------------------------------------------------------------------

void Eau::Reset()
{
  vague = 0;
  temps_eau = Wormux::temps.Lit();
  temps_montee = TEMPS_AVANT_MONTEE; 
}

//-----------------------------------------------------------------------------

void Eau::Refresh()
{
  if (!actif) return;

  // on rempli le sol avec de l'eau
  if (TEMPS_EAU < (Wormux::temps.Lit() - temps_eau))
  {
    temps_eau = Wormux::temps.Lit();
    vague += AVANCE_VAGUES;
    if (surface.get_width() <= vague) vague=0;
  }
}

//-----------------------------------------------------------------------------

void Eau::Active (bool pactif)
{
  actif = pactif;
  if (actif) {
    temps_eau = Wormux::temps.Lit();
    hauteur_eau = HAUTEUR;
  } else {
    hauteur_eau = 0;
  }
}

//-----------------------------------------------------------------------------

void Eau::Draw()
{
  if (!actif) return;
  uint departY = GetHeighteur();

  int x=-surface.get_width()+vague;
  
  do
  {
    for (uint y=departY; y<monde.GetHeight(); )
    {
      if (y==departY) {
	surface.draw (x, y);
	y+=surface.get_height();
      } else {
        fond.draw (x, y);
	y+=fond.get_height();
      }
    }
    x += surface.get_width();
  } while ((uint)x < monde.GetWidth());
}

//-----------------------------------------------------------------------------

uint Eau::GetHeighteur()
{ 
  if (temps.Horloge_Min() >= temps_montee)
  {
    hauteur_eau+=20;
    temps_montee += PAS_DE_LA_MONTEE;
  }	
  return monde.GetHeight()-hauteur_eau;
}

//-----------------------------------------------------------------------------
bool Eau::EstActive() {return actif;}
//-----------------------------------------------------------------------------
