/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 *  Refresh du vent
 *****************************************************************************/

#include "wind.h"
//----------------------------------------------------------------------------
#include "../include/base.h"
#include "../tool/random.h"
#include "../map/map.h"
#include "../map/maps_list.h"
#include "../game/time.h"
#include "../tool/xml_document.h"
#include "../graphic/graphism.h"
#include "camera.h"
//----------------------------------------------------------------------------

const uint BARRE_LARG = 80;
const uint BARRE_HAUT = 10;
const double force = 50.0; // Force maximale du vent en pixel/(sec*sec)
const uint barre_speed = 20;

//----------------------------------------------------------------------------
namespace Wormux {
Vent vent;
//-----------------------------------------------------------------------------
Vent::Vent()
{
  m_val = 0;
  barre.InitPos (10, 10, BARRE_LARG, BARRE_HAUT);
  barre.InitVal (0, -100, 100);
  barre.border_color = CL_Color::white;
  barre.background_color = CL_Color(255*6/10,255*6/10,255*6/10);
  barre.value_color = CL_Color(255*3/10,255*3/10,255);
  barre.AjouteMarqueur (100, CL_Color::white);
  barre.SetReferenceValue (true, 0);
  ChangeVal();

  m_last_move = temps.Lit();
  m_last_part_mvt = temps.Lit();
  m_last_part_anim = temps.Lit();
  nbr_sprite=0;
}
//-----------------------------------------------------------------------------
Vent::~Vent()
{
  if(nbr_sprite!=0)
  {
    delete []particle;
    delete m_wind_sprite;
  }
}
//-----------------------------------------------------------------------------
void Vent::Init()
{
  //Load wind informations
  std::string nomfich = lst_terrain.TerrainActif().m_directory+"config.xml";
  LitDocXml doc;
  if (!doc.Charge (nomfich)) return;
  xmlpp::Element *wind_xml = LitDocXml::AccesBalise (doc.racine(), "wind");
  LoadXML(wind_xml);  
  LoadXML_Sprite(lst_terrain.TerrainActif().res);
}

//-----------------------------------------------------------------------------
void Vent::Reset()
{
  Init();
  m_last_move = temps.Lit();

  if(nbr_sprite!=0)
  {
    m_last_part_mvt = temps.Lit();
    m_last_part_anim = temps.Lit();
    for(int i=0;i<nbr_sprite;i++)
    {
      particle[i].x = RandomLong(0, monde.GetWidth());
      particle[i].y = RandomLong(0, monde.GetHeight());
      //Mass = mass_mean + or - 25%
      particle[i].mass = (double)mass + ((double)mass*RandomLong(-100, 100))/400.0;

      if(sprite_anim_speed != 0)
        particle[i].frame = RandomLong(0,m_wind_sprite->get_frame_count()-1);
      else
        particle[i].frame = 0;
    }
  }
}
//-----------------------------------------------------------------------------

double Vent::LitVal () const
{ return m_nv_val*force/100.0; }

//-----------------------------------------------------------------------------

void Vent::ChangeVal ()
{
  m_nv_val = RandomLong(-100, 100);
}

//-----------------------------------------------------------------------------

void Vent::DrawParticles()
{
  if(nbr_sprite==0)
    return;

  uint delta_t = temps.Lit() - m_last_part_mvt;
  m_last_part_mvt = temps.Lit();

  double dx,dy;

  dx = ((double)(m_val*force)/100.0 * (double)delta_t * sprite_mvt_speed) / 10000.0;
  for(int i=0;i<nbr_sprite;i++)
  {
    particle[i].x += dx;
    dy = delta_t * particle[i].mass / 150.0;
    particle[i].y += dy;

    m_wind_sprite->set_frame(particle[i].frame);
    m_wind_sprite->draw((int)particle[i].x,(int)particle[i].y);
    if(particle[i].x > monde.GetWidth() && dx>0)
      particle[i].x = -m_wind_sprite->get_width();
    else
    if(particle[i].x < -m_wind_sprite->get_width() && dx<0)
      particle[i].x = monde.GetWidth();

    if(particle[i].y > monde.GetHeight() && dy>0)
      particle[i].y = -m_wind_sprite->get_height();
    else
    if(particle[i].y < -m_wind_sprite->get_height() && dy<0)
      particle[i].y = monde.GetHeight();
  }

  if(m_last_part_anim + sprite_anim_speed<temps.Lit() && sprite_anim_speed!=0)
  {
    for(int i=0;i<nbr_sprite;i++)
    {
      particle[i].frame++;
      if(particle[i].frame >= m_wind_sprite->get_frame_count())
        particle[i].frame=0;
    }
    m_last_part_anim=temps.Lit();
  }
}
//-----------------------------------------------------------------------------
void Vent::Draw ()
{
  if(m_last_move + barre_speed < temps.Lit())
  {
    if(m_val>m_nv_val)
      --m_val;
    else
    if(m_val<m_nv_val)
      ++m_val;
    m_last_move = temps.Lit();
  }
  barre.Actu(m_val);  
  barre.Draw();
}

//-----------------------------------------------------------------------------
void Vent::LoadXML(xmlpp::Element *xml)
{
  //Efface les donnes de la dernire partie
  if(nbr_sprite!=0)
  {
    delete []particle;
    delete m_wind_sprite;
    nbr_sprite = 0;
  }
  if(xml==NULL)
    return;
  LitDocXml::LitUint (xml, "nbr_sprite", nbr_sprite);
  LitDocXml::LitInt (xml, "mass", mass);
  LitDocXml::LitUint (xml, "mvt_speed", sprite_mvt_speed);
  LitDocXml::LitUint (xml, "anim_speed", sprite_anim_speed);
  particle = new WindSprite[nbr_sprite];
}

void Vent::LoadXML_Sprite(CL_ResourceManager* res)
{
  if(nbr_sprite==0)
    return;
  m_wind_sprite = new CL_Sprite("wind_sprite",res);
}

//-----------------------------------------------------------------------------
} //namespace Wormux
//-----------------------------------------------------------------------------
