/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Refresh d'un ver de terre.
 *****************************************************************************/

#include "character.h"
//-----------------------------------------------------------------------------
#include <ClanLib/core.h>
#include <sstream>
#include <iostream>
#include "../game/game.h"
#include "../game/game_mode.h"
#include "../game/game_loop.h"
#include "../game/time.h"
#include "../game/config.h"
#include "../graphic/graphism.h"
#include "../include/constant.h"
#include "../map/camera.h"
#include "../map/map.h"
#include "../sound/jukebox.h"
#include "../tool/random.h"
#include "../tool/math_tools.h"
#include "../tool/geometry_tools.h"
#include "../weapon/suicide.h"
#include "../weapon/crosshair.h"
#include "../weapon/weapon_tools.h"
#include "move.h"
#include "macro.h"
using namespace std;
using namespace Wormux;
//-----------------------------------------------------------------------------

const uint HAUT_FONT_MIX = 13;

// Espacement (entre le nom, le ver, et la barre d'nergie)
const uint ESPACE = 3; // pixels

#ifdef DEBUG

//#define ANIME_VITE

//#define DEBUG_CHG_ETAT

//#define DEBUG_CADRE_AUTOUR_VER

//#define DEBUG_PLACEMENT

//#define NO_POSITION_CHECK

//#define DRAW_HAND_POSITION

#define COUT_DBG0 cout << "[Character " << nom << "]"
#define COUT_DBG COUT_DBG0 " "
#define COUT_PLACEMENT COUT_DBG0 "[Init bcl=" << bcl << "] "
#endif

// Pause pour l'animation
#ifdef ANIME_VITE
  const uint ANIM_PAUSE_MIN = 0*1000;
  const uint ANIM_PAUSE_MAX = 3*1000;
#else
  const uint ANIM_PAUSE_MIN = 5*1000;
  const uint ANIM_PAUSE_MAX = 60*1000;
#endif

// Chute
const double CHUTE_SANS_DEGAT = 320; // vitesse  la fin de la chute
const double COEFF_DEGAT_CHUTE = 4.7; // degat = COEFF*(norme-CHUTE_SANS_DEGAT)

// Barre d'nergie
const uint LARG_ENERGIE = 40;
const uint HAUT_ENERGIE = 6;

//-----------------------------------------------------------------------------

Character::Character () : PhysicalObj("Soldat inconnu", 0.0)
{
  pause_bouge_dg = 0;
  m_team = NULL;
  energy = 100;
  losted_energy = 0;
  desactive = false;
  skin = NULL;
  m_wind_factor = 0.0;
}

//-----------------------------------------------------------------------------

// Signale la mort d'un ver
void Character::SignalDeath()
{
#ifdef DEBUG_CHG_ETAT
  COUT_DBG << "Meurt." << endl;
#endif

  // Plus d'nergie ...
  energy = 0;

  jukebox.PlayProfile(GetTeam().LitProfileSon(),"death");

  // Rotation si ncessaire
  CL_Surface img_trou = suicide.hole_image;

  int dir = GetDirection();
  image = GetSkin().dead.image;
  SetDirection (dir);

  //--- Fait exploser le ver :-) ---

  // Le trou est au centre
  CL_Point trou(GetCenter());

  // Mais l'explosion est dans les pieds :)
  CL_Point explosion(GetCenterX(), GetY()+ GetHeight());
  AppliqueExplosion (explosion, trou, img_trou, suicide.cfg(), NULL);

  // Change le rectangle de test
  int x = GetCenterX(), y=GetCenterY();
  SetTestRect (GetSkin().dead.test_dx, 
		  GetSkin().dead.test_dx, 
		  GetSkin().dead.test_top, 
		  GetSkin().dead.test_bottom);
  SetSize (image.get_width(), image.get_height());
  SetXY (x - GetWidth()/2, y - GetHeight()/2);

  assert (m_alive == DEAD);
  assert (IsDead());

  // Signale sa mort
  jeu.SignalCharacterDeath (this);
}

//-----------------------------------------------------------------------------

void Character::SignalDrowning()
{
  energy = 0;
  //Desactive();
  //m_type = objINSENSIBLE;
  image = GetSkin().drowned.image;
}

//-----------------------------------------------------------------------------

// Si un ver devient un fantome, il meurt ! Signale sa mort
void Character::SignalGhostState (bool was_dead)
{
  // Dsactive le ver
  Desactive();

#ifdef DEBUG_CHG_ETAT
  COUT_DBG << "Fantome." << endl;
#endif

  // Signale la mort du ver
  if (!was_dead) jeu.SignalCharacterDeath (this);
}

//-----------------------------------------------------------------------------

void Character::SetDirection (int nv_direction)
{ 
  image.set_scale (nv_direction, 1); 
  if (GetSkin().anim.utilise) anim.image.set_scale (nv_direction, 1);
}

//-----------------------------------------------------------------------------

void Character::DrawEnergyBar(int dy) const
{
  energy_bar.DrawXY (
    GetCenterX()-energy_bar.GetWidth()/2, 
	GetY()+dy);
}

//-----------------------------------------------------------------------------

void Character::DrawName (int dy) const
{
  const int x =  GetCenterX();
  const int y = GetY()+dy;
  police_mix.EcritCentre (x,y,m_name);
}

//-----------------------------------------------------------------------------

void Character::SetEnergyDelta (int delta)
{
  // Si le ver est dj mort, on n'y touche pas
  if (IsDead()) return;

  uint sauve_energie = energy;
  uchar R,V,B;

  // Change l'nergie
  energy = BorneLong((int)energy +delta, 0, mode_jeu.ver.energie_max);
  energy_bar.Actu (energy);
    
  // Choix de la coul de la barre d'nergie
  if (70 < energy) {
    V = 255;
    R = 0;
  } else if (50 < energy) {
    V = 255;
    R = 255;
  } else if (20 < energy) {
    V = 128;
    R = 255;
  } else {
    V = 0;
    R = 255;
  }
  B = 0;

  energy_bar.value_color = CL_Color(R,V,B);

  // Calcule la perte d'nergie
  if (delta < 0)
  {
    
    losted_energy += (int)energy - (int)sauve_energie;
    if ( losted_energy < 33 )
      jukebox.PlayProfile(GetTeam().LitProfileSon(), "injured_light");
    else if ( losted_energy < 66 )
      jukebox.PlayProfile(GetTeam().LitProfileSon(), "injured_medium");
    else 
      jukebox.PlayProfile(GetTeam().LitProfileSon(), "injured_high");
  }
  else 
    losted_energy = 0;

  // "Friendly fire !!"
  if ( (&ActiveCharacter() != this) && (&ActiveTeam() == m_team) )
    jukebox.PlayProfile(GetTeam().LitProfileSon(), "friendly_fire");

  // Dead character ?
  if (energy == 0) Die();
}

//-----------------------------------------------------------------------------

void Character::Draw()
{
  bool dessine_perte = (losted_energy != 0);
  if ((&ActiveCharacter() == this) 
      && (boucle_jeu.LitEtat() != jeuPAUSE_FIN_TOUR)
      && (boucle_jeu.LitEtat() != jeuANIM_FIN_TOUR))
    dessine_perte = false;

  // Il est parti dans un autre monde ?
  if (!IsActive()) return;

  // Dead character ?
  if (IsDead() || IsDrowned()) 
  {
    int x = GetX(), y = GetY();
    if (GetDirection()<0) x += GetWidth();
    image.update();
    image.draw(x,y);
 #if defined(DEBUG_CADRE_AUTOUR_VER)
  CL_Display::draw_rect (CL_Rect(GetX(), GetY(),
				 GetX()+image.get_width(), 
				 GetY()+image.get_height()),
			 CL_Color::blue);
  CL_Display::draw_rect (GetTestRect(), CL_Color::red);
#endif
   return;
  }

  int dy=-ESPACE;

  // Affiche la barre d'energie
  bool est_ver_actif = (this == &ActiveCharacter());
  bool affiche_energie = config.affiche_energie_ver;
  affiche_energie &= !est_ver_actif || (boucle_jeu.LitEtat() != jeuVER_JOUE);
  affiche_energie |= dessine_perte;
  if (affiche_energie)
  { 
    dy -= HAUT_ENERGIE; 
    DrawEnergyBar (dy); 
    dy -= ESPACE; 
  }

  // Affiche le nom
  if (config.affiche_nom_ver && !est_ver_actif) 
  { 
    dy -= HAUT_FONT_MIX;
    DrawName (dy);
    dy -= ESPACE; 
  }

  // Dessine la perte d'nergie ?
  if (dessine_perte)
  {
    ostringstream ss;
    ss << losted_energy;
    dy -= HAUT_FONT_MIX;
    police_mix.EcritCentre (GetX() +GetWidth()/2, GetY()+dy, ss.str());    
  }

  // Dessine l'image du ver
  int x = GetX();
  int y = GetY();
  if (GetDirection()<0) x += GetWidth();
  image.draw (x,y);

#ifdef DRAW_HAND_POSITION
  int a,b,c=3;
  GetHandPosition (a,b);
  CL_Display::draw_line (a-c,b-c, a+c,b+c, CL_Color::blue);
  CL_Display::draw_line (a+c,b-c, a-c,b+c, CL_Color::blue);
#endif

  // Dessine l'animation si ncessaire
  if (anim.draw) 
  {
    anim.image.draw (x,y);
  }

#if defined(DEBUG_CADRE_AUTOUR_VER)
  CL_Display::draw_rect (CL_Rect(GetX(), GetY(),
				 GetX()+image.get_width(), 
				 GetY()+image.get_height()),
			 CL_Color::blue);
  CL_Display::draw_rect (GetTestRect(), CL_Color::red);
#endif
}

//-----------------------------------------------------------------------------

void Character::Saute ()
{
#ifdef DEBUG_CHG_ETAT
  COUT_DBG << "Saute." << endl;
#endif

  jukebox.PlayProfile(ActiveTeam().LitProfileSon(), "jump");

  // Initialise la force
  double angle = Deg2Rad(mode_jeu.ver.angle_saut);
  if (GetDirection() == 1)
    NewStrength (mode_jeu.ver.force_saut, angle);
  else
    NewStrength (mode_jeu.ver.force_saut, InverseAngle(angle));
}

//-----------------------------------------------------------------------------

void Character::SuperSaut ()
{
#ifdef DEBUG_CHG_ETAT
  COUT_DBG << "SuperSaut." << endl;
#endif

  jukebox.PlayProfile(ActiveTeam().LitProfileSon(), "superjump");

  // Initialise la force
  double angle = Deg2Rad(mode_jeu.ver.angle_super_saut);
  if (GetDirection() == 1)
    NewStrength (mode_jeu.ver.force_super_saut, angle);
  else
    NewStrength (mode_jeu.ver.force_super_saut, InverseAngle(angle));
}

//-----------------------------------------------------------------------------

// Refresh du ver (actualisation des donnees)
void Character::Refresh()
{
  if (desactive) return;

  PhysicalEngine ();

  // Refresh de l'animation
  if (GetSkin().anim.utilise) 
  {
    // C'est le dbut d'une animation ?
    if (!anim.draw && (anim.time < temps.Lit())) 
    {
      anim.image.restart();
      anim.draw = true;
    }

    // Animation active
    if (anim.draw)
    {
      anim.image.update();
      if (anim.image.is_finished())
      {
        anim.draw = false;
        anim.time  = RandomLong (ANIM_PAUSE_MIN, ANIM_PAUSE_MAX);
        anim.time += temps.Lit();
      }
    }
  }
}

//-----------------------------------------------------------------------------

// Se prepare a un nouveau tour
void Character::PrepareTour ()
{
  losted_energy = 0;
  pause_bouge_dg = temps.Lit();
}

//-----------------------------------------------------------------------------

const Team& Character::GetTeam() const
{
  assert (m_team != NULL);
  return *m_team;
}

//-----------------------------------------------------------------------------

Team& Character::TeamAccess()
{
  assert (m_team != NULL);
  return *m_team;
}

//-----------------------------------------------------------------------------

bool Character::MouvementDG_Autorise() const
{
  if (!IsReady() || FootsInVacuum()) return false;
  return pause_bouge_dg < temps.Lit();
}

//-----------------------------------------------------------------------------

void Character::InitMouvementDG(uint pause)
{
  pause_bouge_dg = temps.Lit()+pause;
}

//-----------------------------------------------------------------------------

// Signale la fin d'une chute
void Character::SignalFallEnding()
{
  // On ne touche pas aux vers morts, paix  leur me :-(
  if (IsDead()) return;

  double norme, degat, vx, vy;
  CalculateSpeed (vx, vy);
  norme = Norme (vx, vy);
  if (CHUTE_SANS_DEGAT < norme && vy>0.0)
  {
    norme -= CHUTE_SANS_DEGAT;
    degat = norme/COEFF_DEGAT_CHUTE;
    SetEnergyDelta (-(int)degat);
    jeu.autorise_action = false;
    boucle_jeu.ChangeEtat (jeuPAUSE_FIN_TOUR);
  }
}

//-----------------------------------------------------------------------------

int Character::GetDirection() const 
{ 
  float x,y;
  image.get_scale(x,y);
  return (x<0)?-1:1;
}

//-----------------------------------------------------------------------------

bool Character::IsActive() const 
{ return !desactive; }

//-----------------------------------------------------------------------------

// Dsactive le ver
void Character::Desactive()
{
  desactive = true;
}

//-----------------------------------------------------------------------------

// Ractive le ver
void Character::Reactive()
{
  assert (!IsGhost());
  desactive = false;
}

//-----------------------------------------------------------------------------

// Accs  l'avatar
const Skin& Character::GetSkin() const
{
  assert (skin != NULL);
  return *skin;
}

//-----------------------------------------------------------------------------

Skin& Character::AccessSkin()
{
  assert (skin != NULL);
  return *skin;
}

//-----------------------------------------------------------------------------

void Character::FrameImageSuivante()
{
  m_image_frame++;
  if (image.get_frame_count()-1 < (int)(m_image_frame/m_frame_repetition)) 
    m_image_frame = 0;
  image.set_frame (m_image_frame/m_frame_repetition);

  static CL_SoundBuffer_Session *step_session = NULL;

  bool play = (step_session == NULL) || (!step_session -> is_playing());

  if (play)
    jukebox.PlayProfile(m_team->LitProfileSon(), "step", false, &step_session);
}

//-----------------------------------------------------------------------------

void Character::Init() {}

//-----------------------------------------------------------------------------

void Character::InitTeam (Team *ptr_equipe, const string &name, 
			  Skin* pskin)
{
  SetMass (mode_jeu.ver.masse);
  m_team = ptr_equipe;
  m_name = name;
  skin = pskin;

  // Animation ?
  anim.draw = false;
  if (GetSkin().anim.utilise)
  { 
    anim.draw = true;
    anim.image = GetSkin().anim.image;
  }

  // Energie
  energy_bar.InitVal (energy, 0, 100);
  energy_bar.InitPos (0,0, LARG_ENERGIE, HAUT_ENERGIE);
  energy_bar.border_color = CL_Color::black;
  energy_bar.background_color = CL_Color::dimgray;
}

//-----------------------------------------------------------------------------

void Character::Reset() 
{
  // Reset de l'tat du ver
  desactive = false;
  Ready();

  //  Reset de l'image et les dimensions
  image = GetSkin().walking.image;
  SetSize (image.get_width(), image.get_height());
  SetTestRect (GetSkin().walking.test_dx, 
		  GetSkin().walking.test_dx,
		  GetSkin().walking.test_top, 
		  GetSkin().walking.test_bottom);

  // Animation ?
  if (anim.draw)
  { 
    anim.image.finish();
    anim.time  = RandomLong (ANIM_PAUSE_MIN, ANIM_PAUSE_MAX);
    anim.time += temps.Lit();    
  }

  // Initialise l'image
  SetDirection( RandomBool()?1:-1 );
  image.set_frame ( RandomLong(0, image.get_frame_count()-1) );
  m_image_frame = image.get_current_frame();
  m_frame_repetition = GetSkin().walking.repetition_frame;

  // Energie
  energy = mode_jeu.ver.energie_init-1;
  SetEnergyDelta (1);
  losted_energy = 0;

  // Initialise la position
  uint bcl=0;
  bool pos_ok;
  do
  {
    // Vrifie qu'on ne tourne pas en rond
    FORCE_ASSERT (++bcl < NBR_BCL_MAX_EST_VIDE);

    // Objet physique dans l'tat prt
    pos_ok = true;
    desactive = false;
    Ready();

    // Prend des coordonnes au hasard
    uint x = RandomLong(0, monde.GetWidth() -GetWidth());
    uint y = RandomLong(0, monde.GetHeight() -GetHeight());
#ifdef DEBUG_PLACEMENT
    COUT_PLACEMENT << "x0="<<x<<", y="<<y<< endl;
#endif
    SetXY (x, y);

#ifndef NO_POSITION_CHECK
    pos_ok &= !IsGhost() && IsInVacuum(0,0);
#ifdef DEBUG_PLACEMENT
    if (!pos_ok) COUT_PLACEMENT << "Placement XY manqu." << endl;
#endif
    if (!pos_ok) continue;

    // Chute directe pour le sol
    DirectFall ();
    pos_ok &= !IsGhost();
    pos_ok &= !IsDrowned();
#ifdef DEBUG_PLACEMENT
    if (!pos_ok) COUT_PLACEMENT << "Fantome en tombant." << endl;
#endif
    if (!pos_ok) continue;

    // Vrifie que le ver ne fois pas trop prs de ses voisins
    POUR_TOUS_VERS_VIVANTS(it_equipe,ver) if (&(*ver) != this)
    {
      double dst = Distance(ver -> GetCenter(), GetCenter());
      if (dst < monde.dst_min_entre_vers) pos_ok = false;
    }

    // La position est bonne ?
    pos_ok &= !IsGhost() & IsInVacuum(0,0);
#ifdef DEBUG_PLACEMENT
    if (!pos_ok) COUT_PLACEMENT << "Placement final manqu." << endl;
#endif

#endif // of #ifndef NO_POSITION_CHECK
  } while (!pos_ok);
  assert (!IsDead());
  Ready();
}

//-----------------------------------------------------------------------------

uint Character::GetEnergy() const 
{
  assert (!IsDead());
  return energy; 
}

//-----------------------------------------------------------------------------

// Hand position
void Character::GetHandPosition (int &x, int &y) {
  int frame = image.get_current_frame();
  skin_translate_t hand = GetSkin().walking.hand_position.at(frame);
  y = GetY() +hand.dy;
  if (GetDirection() == 1) 
    x = GetX() +hand.dx;
  else
    x = GetX() +GetWidth() -hand.dx;
}

//-----------------------------------------------------------------------------
