/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Skin : gestion des diffrents aspects pour les vers. Spcifications :
 * - image (animation),
 * - position du viseur pour les diffrentes armes,
 * - taille,
 * - ...
 *
 * Chaque avatar diffrent a une seule instance. Un ver possde seulement un
 * pointeur vers une de ces instances.
 *****************************************************************************/

#include "skin.h"
//-----------------------------------------------------------------------------
#include "../game/config.h" // DATADIR
#include "../graphic/graphism.h"
#include "../tool/i18n.h"
#include "../tool/file_tools.h"
#include "../tool/string_tools.h"
#include <dirent.h>
#include <sys/stat.h>
using namespace Wormux;
using namespace std;

const std::string NOMFICH_CONFIG = "config.xml";
//-----------------------------------------------------------------------------
typedef std::map<std::string, Skin>::value_type paire_avatar;
std::map<std::string, Skin> skins_list;
//-----------------------------------------------------------------------------
CfgSkin_Dead::CfgSkin_Dead() { Reset(); }
void CfgSkin_Dead::Reset()
{
  image = CL_Sprite(); // NULL sprite
  test_bottom = 0;
  test_top = 0;
  test_dx = 0;
}
//-----------------------------------------------------------------------------
CfgSkin_Walking::CfgSkin_Walking() { Reset(); }
void CfgSkin_Walking::Reset()
{
  CfgSkin_Dead::Reset();
  repetition_frame = 1;
  hand_position.clear();
}
//-----------------------------------------------------------------------------
CfgSkin_Anim::CfgSkin_Anim() { Reset(); }
void CfgSkin_Anim::Reset()
{
  utilise = false;
  pos.dx = 0;
  pos.dy = 0;
}
//-----------------------------------------------------------------------------

Skin::Skin() { Reset(); }

//-----------------------------------------------------------------------------

void Skin::Reset()
{
  walking.Reset();
  dead.Reset();
  anim.Reset();
  anim.utilise = false;
  anim.vitesse = 100;
}

//-----------------------------------------------------------------------------

bool Skin::Charge (const std::string &nom, const std::string &repertoire)
{
  try
  {
    std::string nomfich;
    LitDocXml doc;

    cout << "o " << Format(_("Load the skin %s"), nom.c_str()) << endl;

    Reset();

    // Load XML file
    nomfich = repertoire+"config.xml";
    CL_ResourceManager res(nomfich, false);
    if (!doc.Charge (nomfich)) return false;

    // Load images
    walking.image = CL_Sprite("walking", &res);
    dead.image = CL_Sprite("dead", &res);

    if(LitDocXml::Access (doc.racine(), "sprite", "drowned") != NULL)
      drowned.image = CL_Sprite("drowned", &res);
    else
      drowned.image = CL_Sprite("dead", &res);

    // <animation>
    anim.utilise = 
      (LitDocXml::Access (doc.racine(), "sprite", "animation") != NULL);
    if (anim.utilise) {
      anim.image = CL_Sprite("animation", &res);
      anim.image.set_play_loop (false);
    }

    // Load XML datas
    if (!TraiteXml (doc.racine())) return false;
  }
  catch (const xmlpp::exception &e)
  {
    std::cout << Format(_("Error during skin loading (%s) :"),nom.c_str()) << std::endl
	      << e.what() << std::endl;
    return false;
  }
  return true;
}

//-----------------------------------------------------------------------------

void Skin::Xml_LitRectTest (CfgSkin_Dead &img, xmlpp::Element *elem)
{
  img.test_dx = img.test_top = img.test_bottom = 0;
  if (elem == NULL) return;
  xmlpp::Element *collision_rect = LitDocXml::AccesBalise (elem, "collision_rect");
  if (collision_rect == NULL) return;
  LitDocXml::LitAttrUint (collision_rect, "dx", img.test_dx);
  LitDocXml::LitAttrUint (collision_rect, "top", img.test_top);
  LitDocXml::LitAttrUint (collision_rect, "bottom", img.test_bottom);
}

//-----------------------------------------------------------------------------

void Skin::Xml_ReadHandPosition(xmlpp::Element *root) {
  int n = walking.image.get_frame_count();
  skin_translate_t pos = {0,0};
  walking.hand_position.clear();
  walking.hand_position.assign (n, pos);

  xmlpp::Node::NodeList nodes = root -> get_children("hand");
  xmlpp::Node::NodeList::iterator 
    it=nodes.begin(),
    end=nodes.end();

  for (; it != end; ++it) {
    xmlpp::Element *elem = dynamic_cast<xmlpp::Element*> (*it);
    assert (elem != NULL);

    if (!LitDocXml::LitAttrInt(elem, "x", pos.dx)) continue;
    if (!LitDocXml::LitAttrInt(elem, "y", pos.dy)) continue;

    std::string frame_str;
    if (!LitDocXml::LitAttrString(elem, "frame", frame_str)) continue;

    if (frame_str == "*") {
      walking.hand_position.assign (n, pos);	
    } else {
      int frame;
      if (!str2int (frame_str, frame)) continue;
      --frame;
      if (frame < 0) continue;
      if (n<frame) continue;
      walking.hand_position.at(frame) = pos;
    }
  }
}

//-----------------------------------------------------------------------------

bool Skin::TraiteXml (xmlpp::Element *xml)
{
  std::string txt;

  // <walking>
  xmlpp::Element *elem = LitDocXml::Access (xml, "sprite", "walking");
  if (elem != NULL) {
    Xml_LitRectTest (walking, elem);

    // <walking><wormux repetition="xxx">
    xmlpp::Element *animation = LitDocXml::AccesBalise (elem, "wormux");
    if (animation != NULL) {
      LitDocXml::LitAttrUint (animation, "repetition", 
			      walking.repetition_frame);
    }

    Xml_ReadHandPosition(elem);
  }
  
  // <dead>
  elem = LitDocXml::Access (xml, "sprite", "dead");
  if (elem != NULL) Xml_LitRectTest (dead, elem);

  return true;
}

//-----------------------------------------------------------------------------

void InitSkins()
{
  Skin skin;
  std::string repertoire = config.data_dir+"skin/";
  struct dirent *fichier;
  DIR *dir;

  dir = opendir(repertoire.c_str());
  if (dir == NULL) 
    Erreur (Format(_("Impossible to open skins directory (%s) !"),
		   repertoire.c_str()));

  // Parcours le rpertoire des avatars
  while ((fichier = readdir(dir)) != NULL)
  {
    std::string nomfich_complet = repertoire+fichier -> d_name;
    struct stat stat_file;

    // On a trouv un rpertoire ?
    if ((fichier -> d_name[0] != '.')
	&& (stat(nomfich_complet.c_str(), &stat_file) == 0)
	&& (S_ISDIR(stat_file.st_mode)))
    {
      std::string rep_avatar = nomfich_complet+'/';

      // Si on arrive  le charger correctement, l'ajoute  la liste
      if (FichierExiste(rep_avatar+NOMFICH_CONFIG)
	  && skin.Charge (fichier -> d_name, rep_avatar))
      {
	skins_list.insert (paire_avatar(fichier -> d_name, skin));
      }
    }
  }

  closedir (dir);
}

//-----------------------------------------------------------------------------
