/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Skin : gestion des diffrents aspects pour les vers. Spcifications :
 * - image (animation),
 * - position du viseur pour les diffrentes armes,
 * - taille,
 * - ...
 *
 * Chaque avatar diffrent a une seule instance. Un ver possde seulement un
 * pointeur vers une de ces instances.
 *****************************************************************************/

#ifndef SKIN_H
#define SKIN_H
//-----------------------------------------------------------------------------
#include "../include/base.h"
#include "../tool/xml_document.h"
#include "../graphic/animation.h"
#include <ClanLib/display.h>
#include <ClanLib/core.h>
#include <map>
#include <string>
//-----------------------------------------------------------------------------

typedef struct { int dx,dy; } skin_translate_t;

class CfgSkin_Dead
{
public:
  CL_Sprite image;
  uint test_dx, test_top, test_bottom;
public:
  CfgSkin_Dead();
  void Reset();
};

//-----------------------------------------------------------------------------

class CfgSkin_Walking : public CfgSkin_Dead
{
public:
  uint repetition_frame;
  std::vector<skin_translate_t> hand_position;
public:
  CfgSkin_Walking();
  void Reset();
};

//-----------------------------------------------------------------------------

class CfgSkin_Anim
{
public:
  bool utilise;
  CL_Sprite image;
  skin_translate_t pos;
  uint vitesse;

public:
  CfgSkin_Anim();
  void Reset();
};

//-----------------------------------------------------------------------------

class Skin
{
public:
  // Reprsentation du ver
  CfgSkin_Walking walking;
  CfgSkin_Dead dead;
  CfgSkin_Dead drowned;

  // Animation (yeux qui clignotent ?)
  CfgSkin_Anim anim;

public:
  Skin();
  void Reset();
  bool Charge (const std::string &nom, const std::string &repertoire);

protected:
  bool TraiteXml (xmlpp::Element *xml);
  void Xml_LitRectTest (CfgSkin_Dead &img, xmlpp::Element *elem);
  void Xml_ReadHandPosition(xmlpp::Element *elem);
};

//-----------------------------------------------------------------------------

// Liste des avatars
extern std::map<std::string, Skin> skins_list;

// Initialisation des avatars
void InitSkins();

//-----------------------------------------------------------------------------
#endif
