/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Une quipe de vers.
 *****************************************************************************/

#ifndef EQUIPE_H
#define EQUIPE_H
//-----------------------------------------------------------------------------
#include "../include/base.h"
#include "../tool/xml_document.h"
#include "../weapon/crosshair.h"
#include "../weapon/weapon.h"
#include "../graphic/image.h"
#include "character.h"
#include "team_energy.h"
#include <ClanLib/display.h>
#include <string>
#include <list>
#include <map>
//-----------------------------------------------------------------------------

class Team
{
public:
  typedef std::list<Character>::iterator iterator;
  typedef std::list<Character>::const_iterator const_iterator;

  // This is public for the bonus_box...
  std::map<std::string, int> m_nb_bullets;

  // Autres
  Viseur viseur;
  CL_Surface ecusson;
  CL_Point sauve_camera;
  bool camera_est_sauve;
  TeamEnergy energie;
 
private:
  std::string m_nom;
  std::string m_nomfich;
  std::string m_profile_son;
  std::list<Character> vers;
  iterator vers_fin;
  iterator ver_actif;
  Arme *arme_active;
  bool ChargeDonnee (int nbr_ver, xmlpp::Element *xml, CL_ResourceManager *res);

public:
  // Initialisation
  Team ();
  bool Init (int nbr_ver, 
	     const std::string &name, 
	     const std::string &directory);
  void Reset();

  // Passe au ver suivant
  void NextCharacter();

  // Select an *ALIVE* character
  void SelectCharacter (iterator index);

  // Prepare/met fin au tour
  void PrepareTour();
  void FinTour();

  // Acces aux vers
  Character& ActiveCharacter();

  void Draw();
  void Update();

  // Change d'arme
  void ChangeArme (Arme * nv_arme);
  uint NbAliveCharacter() const;

  // Accs  l'arme active
  Arme& AccesArme() const { return *arme_active; }
  const Arme& LitArme() const { return *arme_active; }

  // Initialisation de la barre d'nergie de l'quipe
  void InitEnergy (uint max);

  // Actualisation des valeurs la barre d'nergie
  void ActualiseBarreEnergie();

  // Lecture de l'nergie totale de l'quipe
  uint LitEnergie();

  // Acces aux donnes
  const std::string& LitNom() const { return m_nom; }
  const std::string& LitNomfich() const { return m_nomfich; }  
  const std::string& LitProfileSon() const { return m_profile_son; }
  iterator Debut();
  iterator Fin();

  // Nombre de munition pour l'arme en cours
  // (renvoie MUNITION_ILLIMITEES si les munitions sont illimites)
  int ReadNbBullets() const;
  int ReadNbBullets(const std::string &weapon_name) const;
  int& AccessNbBullets();

protected:
  void internal_NextCharacter();
};

//-----------------------------------------------------------------------------
#endif
