/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Refresh des diffrentes quipes.
 *****************************************************************************/

#include "teams_list.h"
//-----------------------------------------------------------------------------
#include "../game/config.h"
#include "../tool/file_tools.h"
#include "../tool/i18n.h"
#include "team_energy.h"
#include <dirent.h>
#include <sys/stat.h>
#include <algorithm>

//-----------------------------------------------------------------------------
TeamsList teams_list;
//-----------------------------------------------------------------------------

TeamsList::TeamsList()
{}

//-----------------------------------------------------------------------------

void TeamsList::NextTeam (bool debut_jeu)
{
  // Fin du tour pour l'quipe active
  if (!debut_jeu) 
  {
    ActiveTeam().FinTour();

    // Passe  l'quipe suivante
    do
    {
      ++m_equipe_active;
      if (m_equipe_active == list.end()) m_equipe_active = list.begin();
    } while (ActiveTeam().NbAliveCharacter() == 0);
  }

  // Prepare le tour d'une equipe
  ActiveTeam().PrepareTour();
  assert (!ActiveCharacter().IsDead());
}

//-----------------------------------------------------------------------------

Team& TeamsList::ActiveTeam()
{
  assert (m_equipe_active != list.end());
  return **m_equipe_active;
}

//-----------------------------------------------------------------------------

void TeamsList::LoadList()
{
  const std::string repertoire = Wormux::config.data_dir+"team/";
  struct dirent *fichier;
  DIR *dir;
  
  dir = opendir(repertoire.c_str());
  if (dir == NULL) 
    Erreur (Format(_("Cannot open teams directory (%s)!"),
		   repertoire.c_str()));

  // Cherche toutes les quipes
  list.clear() ;
  while ((fichier = readdir(dir)) != NULL)
  {
    std::string nomfich_complet = repertoire+fichier -> d_name;
    struct stat stat_file;

    // On a trouv un fichier normal ?
    if ((fichier -> d_name[0] != '.')
	&& (stat(nomfich_complet.c_str(), &stat_file) == 0)
	&& (S_ISDIR(stat_file.st_mode)))
    {
      // Ajoute l'quipe  la liste
      Team nv_equipe;
      full_list.push_back(nv_equipe);

      // Charge les donnes de l'quipe, 
      // en cas d'erreur retire l'quipe de la liste
      std::string rep_equipe = nomfich_complet + '/';
      if (!full_list.back().Init (-1, fichier -> d_name, rep_equipe)) 
      {
	full_list.pop_back();
      }
    }
  }  
  closedir (dir);

  // On a au moins deux quipes ?
  if (full_list.size() < 2)
    Erreur(_("You need at least two valid teams !"));

  // Slection bidon
  std::list<uint> nv_selection;
  nv_selection.push_back (0);
  nv_selection.push_back (1);
  ChangeSelection (nv_selection);
}

//-----------------------------------------------------------------------------

void TeamsList::Reset()
{
  m_equipe_active = list.begin();

  // Commence par dsactiver tous les vers
  iterator it=list.begin(), fin=list.end();
  for (; it != fin; ++it) 
  {
    Team::iterator ver=(**it).Debut(), dernier_ver=(**it).Fin();
    for (; ver != dernier_ver; ++ver) (*ver).Desactive();
  } 

  // Reset de toutes les quipes
  for (it=list.begin(); it != fin; ++it) (**it).Reset();
}

//-----------------------------------------------------------------------------

Team *TeamsList::FindByFilename (const std::string &nomfich, int &pos)
{
  full_iterator it=full_list.begin(), fin=full_list.end();
  int i=0;
  for (; it != fin; ++it, ++i) 
  {
    if (it -> LitNomfich() == nomfich) 
    {
      pos = i;
      return &(*it);
    }
  }
  pos = -1;
  return NULL;
}

//-----------------------------------------------------------------------------

Team *TeamsList::FindByIndex (uint index)
{
  full_iterator it=full_list.begin(), fin=full_list.end();
  uint i=0;
  for (; it != fin; ++it, ++i) 
  {
    if (i == index) return &(*it);
  }
  assert (false);
  return NULL;
}

//-----------------------------------------------------------------------------

void TeamsList::InitList (const std::list<std::string> &liste_nom)
{
  std::list<std::string>::const_iterator it=liste_nom.begin(), fin=liste_nom.end();
  selection.clear();
  list.clear();
  for (; it != fin; ++it)
  {
    int pos;
    Team *equipe = FindByFilename (*it, pos);
    if (equipe != NULL) {
      selection.push_back (pos);
      list.push_back (equipe);
    } else {
      std::cout << "Equipe " << *it << " introuvable :-(" << std::endl;
    }
  }
  m_equipe_active = list.begin();
}

//-----------------------------------------------------------------------------

void TeamsList::InitEnergy ()
{
  //On cherche l'quipe avec le plus d'nergie pour fixer le niveau max
  //(arrive dans le cas d'quipe n'ayant pas le mme nombre de vers)
  iterator it=list.begin(), fin=list.end();
  uint max = 0;
  for (; it != fin; ++it)
  {
    if( (**it).LitEnergie() > max)
      max = (**it).LitEnergie();
  }

  //Initialisation de la barre d'nergie de chaque quipe
  it=list.begin();
  uint i = 0;
  for (; it != fin; ++it)
  {
    (**it).InitEnergy (max);
    ++i;
  }

  //Calcul du classement initial
  it=list.begin();
  for (; it != fin; ++it)
  {
    uint classement = 0;
    iterator it2=list.begin();
    for (; it2 != fin; ++it2)
    {
      if((it != it2)
      && (**it2).LitEnergie() > (**it).LitEnergie() )
        ++classement;
    }
    (**it).energie.classement_tmp = classement;
  }
  it=list.begin();
  for (; it != fin; ++it)
  {
    uint classement = (**it).energie.classement_tmp;
    iterator it2=list.begin();
    for (it2 = it; it2 != fin; ++it2)
    {
      if((it != it2)
      && (**it2).LitEnergie() == (**it).LitEnergie() )
        ++classement;
    }
    (**it).energie.FixeClassement(classement);
  }
}

//-----------------------------------------------------------------------------

void TeamsList::RefreshEnergy ()
{
  //Dans l'ordre des priorits :
  // - Terminer l'opration en cours
  // - On change la valeur de l'nergie
  // - On change le classement
  // - On prpare les jauges  l'venement suivant
  
  iterator it=list.begin(), fin=list.end();
  uint status;

  bool en_attente = true; // Toute les jauges sont en attente

  for (; it != fin; ++it) {
    if( (**it).energie.status != EnergieStatusAttend)
    {
      en_attente = false;
      break;
    }
  }

  //Une des jauge xcute un ordre?
  if(!en_attente)
  {
    status = EnergieStatusOK;

    //Une des jauges change de valeur?
    for (it=list.begin(); it != fin; ++it) {
      if( (**it).energie.status == EnergieStatusValeurChange) {
        status = EnergieStatusValeurChange;
        break;
      }
    }
  
    //Une des jauges change de classement?
    for (it=list.begin(); it != fin; ++it) {
      if( (**it).energie.status == EnergieStatusClassementChange
      && ((**it).energie.EstEnMouvement() || status == EnergieStatusOK)) {
        status = EnergieStatusClassementChange;
        break;
      }
    }
  }
  else {
    //Les jauges sont toutes en attente
    //->on les met OK pour un nouvel ordre
    status = EnergieStatusOK;
  }

  // On recopie l'ordre a donner aux jauges
  if(status != EnergieStatusOK || en_attente)
  {
    it=list.begin();
    for (; it != fin; ++it) {
      (**it).energie.status = status;
    }
  }

  // Actualisation des valeurs (pas d'actualisation de l'affichage)
  for (it=list.begin(); it != fin; ++it)
  {
    (**it).ActualiseBarreEnergie();
    RefreshSort();
  }
}
//-----------------------------------------------------------------------------

void TeamsList::RefreshSort ()
{
  iterator it=list.begin(), fin=list.end();
  uint classement;
  
  //Cherche le classement sans tenir comte des galits
  it=list.begin();
  for (; it != fin; ++it)
  {
    classement = 0;
    iterator it2=list.begin();
    for (; it2 != fin; ++it2)
    {
      if((it != it2)
      && (**it2).LitEnergie() > (**it).LitEnergie() )
        ++classement;
    }
    (**it).energie.classement_tmp = classement;
  }

  //Rglage des galits
  it=list.begin();
  for (; it != fin; ++it)
  {
    classement = (**it).energie.classement_tmp;
    iterator it2=list.begin();
    for (it2 = it; it2 != fin; ++it2)
    {
      if((it != it2)
      && (**it2).LitEnergie() == (**it).LitEnergie() )
        ++classement;
    }
    (**it).energie.NouveauClassement(classement);
  }
}

//-----------------------------------------------------------------------------

void TeamsList::ChangeSelection (const std::list<uint>& nv_selection)
{
  selection = nv_selection;

  selection_iterator it=selection.begin(), fin=selection.end();
  list.clear();
  for (; it != fin; ++it) list.push_back (FindByIndex(*it));
  m_equipe_active = list.begin();
}

//-----------------------------------------------------------------------------

bool TeamsList::IsSelected (uint index)
{
  selection_iterator pos = std::find (selection.begin(), selection.end(), index);
  return pos != selection.end();
}

//-----------------------------------------------------------------------------

Team& ActiveTeam()
{ return teams_list.ActiveTeam(); }

//-----------------------------------------------------------------------------

Character& ActiveCharacter()
{ return ActiveTeam().ActiveCharacter(); }

//-----------------------------------------------------------------------------
