/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Outils mathmatiques.
 *****************************************************************************/

#ifndef MATH_TOOLS_H
#define MATH_TOOLS_H
//-----------------------------------------------------------------------------
#include "../include/base.h"
#include "../object/physical_obj.h"
#include <ClanLib/core.h>
#include <string>
//-----------------------------------------------------------------------------

// Nombre (rel) considr comme nul
extern const double EPS_ZERO;

// Norme d'un vecteur
double Norme (double x, double y);

// Borne une valeur dans l'intervale [min;max]
long BorneLong (const long &valeur, const long &min, const long &max);
double BorneDouble (const double &valeur, const double &min, 
		    const double &max);

// Calcule l'angle en radian du point M dans le repre de centre O
// Pour O=(0,0) :
// - M=(10,10) -> PI/4 (0.78)
// - M=(0,10) -> PI/2 (1.57)
// - M=(-10,10) -> 3*PI/4 (2.35)
// - M=(10,-10) -> -PI/4 (-0.78)
// - M=O -> 0
double CalculeAngle (const CL_Point &O, const CL_Point &M);

// Inverse un angle par rapport  l'axe vertical
double InverseAngle (const double &angle);

// Conversion degr en radian
double Deg2Rad (int degre);

// Valeur absolue
double AbsReel (const double x);
bool EgalZero (const double x);

// Contertit un double en int en arrondissant  la valeur la plus proche
int Arrondit(double x);
//-----------------------------------------------------------------------------
#endif
