/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Arme bazooka : projette une roquette avec un angle et une force donne.
 *****************************************************************************/

#include "../weapon/auto_bazooka.h"
//-----------------------------------------------------------------------------
#include "../game/time.h"
#include "../game/config.h"
#include "../interface/mouse.h"
#include "../map/map.h"
#include "../team/teams_list.h"
#include "../interface/interface.h"
#include "../graphic/video.h"
#include "../tool/math_tools.h"
#include "../game/game_loop.h"
#include "../tool/i18n.h"
#include "../graphic/graphism.h"
#include "../map/camera.h"
#include "../weapon/weapon_tools.h"
#include "../interface/game_msg.h"
#include "../object/objects_list.h"
#include "../sound/jukebox.h"
#include "../map/wind.h"
//-----------------------------------------------------------------------------
namespace Wormux {

AutomaticBazooka auto_bazooka;
//-----------------------------------------------------------------------------

//Temps en seconde  partir duquel la roquette se dirige vers la cible
const uint TPS_AV_ATTIRANCE = 1;

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

RoquetteTeteCherche::RoquetteTeteCherche() 
  : ProjectileArme("roquette tete chercheuse")
{
  m_allow_negative_y = true;
  touche_ver_objet = true;
  m_attire = false;
}

//-----------------------------------------------------------------------------

void RoquetteTeteCherche::Tire (int force, 
				int x, int y,
				uint cible_x, uint cible_y)
{
  PrepareTir();

  // Fixe la position de dpart
  SetXY (x,y);

  //On choisit la cible pour la roquette:
  m_cible.x = cible_x;
  m_cible.y = cible_y;
  
  //La roquette n'est pas attire par la cible ds le dpart:
  m_attire = false;
  
  NewSpeedXY (0.0,0.0);
  double angle = ActiveTeam().viseur.LitAngle();
  NewStrength (force, angle);

  temps_debut_tir = Wormux::temps.Lit();
  angle_local=angle;
}

//-----------------------------------------------------------------------------

void RoquetteTeteCherche::Init()
{
  image = CL_Sprite("roquette", &graphisme.weapons);
  SetSize (image.get_width(), image.get_height());

  SetMass (auto_bazooka.cfg().masse);

  // Fixe le rectangle de test
  int dx = image.get_width()/2-1;
  int dy = image.get_height()/2-1;
  SetTestRect (dx, dx, dy, dy);
}

//-----------------------------------------------------------------------------

void RoquetteTeteCherche::Refresh()
{
  if (!is_active) return;

  if (TestImpact()) { SignalCollision(); return; }

  if(m_attire)
  {
    double angle = CalculateMvtAngle();
    image.set_angle (angle *180/M_PI);
  }
  else
  {
    //La roquette tourne sur elle-mme
    angle_local += M_PI / 8;
    if(angle_local > M_PI) angle_local = - M_PI;
    double angle = angle_local;

    image.set_angle (angle *180/M_PI);

    //2 sec aprs avoir t tire, la roquette se dirige vers la cible:
    double tmp = Wormux::temps.Lit() - temps_debut_tir;
    if(tmp>1000 * TPS_AV_ATTIRANCE)
      m_attire = true;
    
  }
}
//-----------------------------------------------------------------------------

void RoquetteTeteCherche::SignalCollision()
{ 
  if (IsGhost())
  {
    messages_jeu.Ajoute (_("The automatic rocket has left the battlefield..."));
  }
  is_active = false; 
}

//-----------------------------------------------------------------------------

void RoquetteTeteCherche::Reset()
{}

//-----------------------------------------------------------------------------

// Choisit les coordonnes de la cible 	 
void RoquetteTeteCherche::SetTarget (int x, int y) 	 
{ 	 
  m_cible.x = x; 	 
  m_cible.y = y; 	 
} 	 

//-----------------------------------------------------------------------------

// Calcule la position d'un point  un moment donn
void RoquetteTeteCherche::CalculateXY (double &nx, double &ny)
{
  double delta_t = (Wormux::temps.Lit() - m_last_move) / 1000.0;
  nx = m_pos.x;
  ny = m_pos.y;
  double acc_x, acc_y;
  m_speed.x += m_strength.x;
  m_speed.y += m_strength.y;
  if (!m_attire) {
    m_speed.x += vent.LitVal() * m_wind_factor * delta_t;
    m_speed.y += config.gravity * delta_t * GetMass() / 5.0;
  }
  nx += m_speed.x * delta_t;
  ny += m_speed.y * delta_t;
  
  //Rinitialisation de la force
  if (m_attire) {
    double angle = CalculeAngle(GetPos(), m_cible);
    double vit  = hypot(m_cible.x - nx, m_cible.y - ny);
    m_strength.x = 1000.0 * delta_t * cos(angle);
    m_strength.y = 1000.0 * delta_t * sin(angle);
  } else {
    m_strength.x = 0;
    m_strength.y = 0;
  }
  m_last_move = Wormux::temps.Lit();
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

AutomaticBazooka::AutomaticBazooka() : Arme("automatic_bazooka")
{
  m_is_active = false;
  cible.choisie = false;
  extra_params = new ExplosiveWeaponConfig();
}

//-----------------------------------------------------------------------------

void AutomaticBazooka::Draw()
{
  Arme::Draw();
  DrawTarget();
}

//-----------------------------------------------------------------------------

bool AutomaticBazooka::Tire ()
{
  m_is_active = true;

  if (!UtiliseMunition()) return false;

  ActiveCharacter().previous_strength = m_strength;
  if (m_strength == max_strength)
  {
    m_strength = 0;
    ExplosionDirecte();
    return true;
  }

  // Initialise le roquette
  int x,y;
  PosArmeXY (x,y);
  roquette.Tire (m_strength, x,y, cible.pos.x,cible.pos.y);
  lst_objets.AjouteObjet (&roquette,true);

  m_strength = 0;
  jukebox.PlayProfile(ActiveTeam().LitProfileSon(), "fire");

  return true;
}

//-----------------------------------------------------------------------------

// Le bazooka explose car il a t pouss  bout !
void AutomaticBazooka::ExplosionDirecte()
{
  CL_Point pos = ActiveCharacter().GetCenter();
  AppliqueExplosion (pos, pos, impact, cfg(), NULL);
}

//-----------------------------------------------------------------------------

void AutomaticBazooka::Explosion()
{
  m_is_active = false;
  
  lst_objets.RetireObjet (&roquette);

  // On fait un trou ?
  if (roquette.IsGhost()) return;

  // Applique les degats et le souffle aux vers
  CL_Point pos = roquette.GetCenter();
  AppliqueExplosion (pos, pos, impact, cfg(), NULL);
}

//-----------------------------------------------------------------------------

void AutomaticBazooka::Refresh()
{
  if(cible.choisie)
    cible.image.draw (cible.pos.x,cible.pos.y);

  if (m_is_active)
  {
    if (!roquette.is_active) Explosion();
  } 
}

//-----------------------------------------------------------------------------

void AutomaticBazooka::Desactive() 
{
  Arme::Desactive();
  cible.choisie = false;
}

//-----------------------------------------------------------------------------

void AutomaticBazooka::Init()
{
  m_name = _("Automatic bazooka");
  roquette.Init();
  m_image = CL_Surface("bazooka", &graphisme.weapons);
  impact = CL_Surface("bazooka_impact", &graphisme.weapons);
  cible.image = CL_Surface("baz_cible", &graphisme.weapons);
  icone = CL_Surface("bazooka_icon", &graphisme.weapons);
}

//-----------------------------------------------------------------------------

void AutomaticBazooka::Reset()
{
}

//-----------------------------------------------------------------------------
bool AutomaticBazooka::CibleActive() const
{ return cible.choisie; }
//-----------------------------------------------------------------------------

bool AutomaticBazooka::ChooseTarget()
{
  cible.pos = souris.GetPosMonde();
  if (monde.EstHorsMonde(cible.pos)) return false;
  DrawTarget();
  cible.choisie = true;
  return false ;
}

//-----------------------------------------------------------------------------

void AutomaticBazooka::DrawTarget()
{
  if(!cible.choisie) { return; }
  cible.image.draw (cible.pos.x - (cible.image.get_width() / 2),
  			cible.pos.y - (cible.image.get_height() / 2));
}

//-----------------------------------------------------------------------------

bool AutomaticBazooka::IsReady() const
{
  return (MunitionSuffisante() && CibleActive());  
}

//-----------------------------------------------------------------------------

ExplosiveWeaponConfig& AutomaticBazooka::cfg()
{ return static_cast<ExplosiveWeaponConfig&>(*extra_params); }

//-----------------------------------------------------------------------------

} // namespace Wormux
