/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Arme grenade  fragmentation(cluster bomb)
 * Explose au bout de quelques secondes 
 *****************************************************************************/

#include "cluster_bomb.h"
//-----------------------------------------------------------------------------
#include <sstream>
#include <math.h>
#include "../game/time.h"
#include "../team/teams_list.h"
#include "../interface/interface.h"
#include "../graphic/video.h"
#include "../tool/math_tools.h"
#include "../game/game_loop.h"
#include "../graphic/graphism.h"
#include "../map/camera.h"
#include "../weapon/weapon_tools.h"
#include "../interface/game_msg.h"
#include "../tool/i18n.h"
#include "../object/objects_list.h"
#include "../sound/jukebox.h"
#include "../tool/random.h"
//-----------------------------------------------------------------------------
namespace Wormux {
LanceCluster lance_cluster;
//-----------------------------------------------------------------------------

#ifdef DEBUG

#define COUT_DBG std::cout << "[ClusterBomb] "

#endif

//-----------------------------------------------------------------------------
extern LanceCluster lance_cluster;
//-----------------------------------------------------------------------------

Cluster::Cluster() : ProjectileArme ("Cluster")
{
  m_allow_negative_y = true;
  touche_ver_objet = true;
}

//-----------------------------------------------------------------------------

void Cluster::Tire (int x, int y)
{
  PrepareTir();
  SetXY(x,y);
}

//-----------------------------------------------------------------------------

void Cluster::Init()
{
  image = CL_Sprite("cluster", &graphisme.weapons);
  SetSize (image.get_width(), image.get_height());

  SetMass (lance_cluster.cfg().masse);

  // Fixe le rectangle de test
  int dx = image.get_width()/2-1;
  int dy = image.get_height()/2-1;
  SetTestRect (dx, dx, dy, dy);
}

//-----------------------------------------------------------------------------

void Cluster::Refresh()
{
  if (!is_active) return;

  double angle = CalculateMvtAngle() * 180/M_PI ;
  image.set_angle (angle);
}

//-----------------------------------------------------------------------------

void Cluster::Draw()
{
  image.draw(GetX(),GetY());
}

//-----------------------------------------------------------------------------

void Cluster::SignalCollision()
{
  is_active = false;
  lst_objets.RetireObjet((PhysicalObj*)this);
  if (IsGhost())
  {
    messages_jeu.Ajoute (_("The rocket left the battlefield..."));
    return;
  }
  AppliqueExplosion (GetPos(), GetPos(), lance_cluster.impact, lance_cluster.cfg(), NULL);
}

//-----------------------------------------------------------------------------

void Cluster::Reset()
{
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

ClusterBomb::ClusterBomb() : ProjectileArme ("cluster_bomb")
{
  m_allow_negative_y = true;
  m_rebound_sound = "weapon/grenade_bounce";
  touche_ver_objet = false;
  m_rebounding = true;
}

//-----------------------------------------------------------------------------

ClusterBomb::~ClusterBomb()
{
  delete []tableau_cluster;
}

//-----------------------------------------------------------------------------

void ClusterBomb::Tire (int force)
{
  PrepareTir();

  // Fixe la position de dpart
  int x,y;
  ActiveTeam().LitArme().PosArmeXY (x, y);
  SetXY (x,y);

  NewSpeedXY (0.0,0.0);
  // Fixe la force de dpart
  double angle = ActiveTeam().viseur.LitAngle();
  NewStrength (force, angle);

#ifdef MSG_DBG
  COUT_DBG << "ClusterBomb::Tire()" << std::endl;
#endif

  // Recupere le moment du dpart
  temps_debut_tir = Wormux::temps.Lit();
}

//-----------------------------------------------------------------------------

void ClusterBomb::Init()
{
  image = CL_Sprite("clusterbomb_sprite", &graphisme.weapons);
  SetSize (image.get_width(), image.get_height());

  SetMass (lance_cluster.cfg().masse);
  m_rebound_factor = double(lance_cluster.cfg().coeff_rebond)/100;

  // Fixe le rectangle de test
  int dx = image.get_width()/2-1;
  int dy = image.get_height()/2-1;
  SetTestRect (dx, dx, dy, dy);

#ifdef MSG_DBG
  COUT_DBG << "ClusterBomb::Init()" << std::endl;
#endif

  tableau_cluster = new Cluster[lance_cluster.cfg().nbr_fragments];
  for(uint i=0;i<lance_cluster.cfg().nbr_fragments;i++)
    tableau_cluster[i].Init();
}

//-----------------------------------------------------------------------------

void ClusterBomb::Refresh()
{
  if (!is_active) return;


#ifdef MSG_DBG
  COUT_DBG << "ClusterBomb::Refresh()" << std::endl;
#endif

  if (IsGhost())
  {
    messages_jeu.Ajoute ("The Cluster Bomb left the battlefield before it could explode.");
    is_active = false;
    return;
  }

  // 5 sec aprs avoir t tire, la grenade explose
  double tmp = Wormux::temps.Lit() - temps_debut_tir;
  if(tmp>1000 * lance_cluster.cfg().tps_avt_explosion)
  {
    double vit_x,vit_y;
    CalculateSpeed(vit_x,vit_y);

    for(uint i=0;i<lance_cluster.cfg().nbr_fragments;i++)
    {
      double angle = (double)RandomLong ((long)0.0, (long)(2.0 * M_PI));
      tableau_cluster[i].Tire(GetX()+(int)(cos(angle) * (double)lance_cluster.cfg().dst_souffle/4),
                              GetY()+(int)(sin(angle) * (double)lance_cluster.cfg().dst_souffle/4));
      tableau_cluster[i].NewSpeedXY(vit_x,vit_y);
      lst_objets.AjouteObjet((PhysicalObj*)&tableau_cluster[i],false);
    }
    is_active = false;
    return;
  }

  if (TestImpact()) { SignalCollision(); return; }

  // rotation de l'image de la grenade...
  double angle = CalculateMvtAngle() * 180/M_PI ;
  image.set_angle (angle);
}

//-----------------------------------------------------------------------------

void ClusterBomb::Draw()
{
  if (!is_active) return;
  if (IsGhost())
  {
    messages_jeu.Ajoute ("The Cluster Bomb left the battlefield before it could explode.");
    is_active = false;
    return;
  }

  image.draw(GetX(),GetY());
  int tmp = lance_cluster.cfg().tps_avt_explosion;
  tmp -= (int)((Wormux::temps.Lit() - temps_debut_tir) / 1000);
  std::ostringstream ss;
  ss << tmp;
  int txt_x = GetX() + GetWidth() / 2;
  int txt_y = GetY() - GetHeight();
  police_mix.EcritCentre (txt_x, txt_y, ss.str());
}

//-----------------------------------------------------------------------------

void ClusterBomb::SignalCollision()
{   
  if (IsGhost())
  {
    messages_jeu.Ajoute ("The Cluster Bomb left the battlefield before it could explode.");
    is_active = false ;
  }
}

//-----------------------------------------------------------------------------

void ClusterBomb::Reset()
{
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

LanceCluster::LanceCluster() : Arme("cluster_bomb")
{
  extra_params = new ClusterBombConfig();
}

//-----------------------------------------------------------------------------

bool LanceCluster::Tire ()
{
  m_is_active = true;

  if (!UtiliseMunition()) return false;

  ActiveCharacter().previous_strength = m_strength;

  // Initialise la grenade
  cluster_bomb.Tire (m_strength);
  camera.ChangeObjSuivi (&cluster_bomb, true, false);
  lst_objets.AjouteObjet (&cluster_bomb, true);
  m_strength = 0;

  jukebox.PlayProfile(ActiveTeam().LitProfileSon(), "fire");

#ifdef MSG_DBG
  COUT_DBG << "LanceCluster::Tire()" << std::endl;
#endif

  return true;
}

//-----------------------------------------------------------------------------

void LanceCluster::Explosion()
{
  m_is_active = false;

#ifdef MSG_DBG
  COUT_DBG << "LanceCluster::Explosion()" << std::endl;
#endif
  
  lst_objets.RetireObjet (&cluster_bomb);

  // On fait un trou ?
  if (cluster_bomb.IsGhost()) return;

  // Applique les degats et le souffle aux vers
  CL_Point pos = cluster_bomb.GetCenter();
  AppliqueExplosion (pos, pos, impact, cfg(), NULL);
}

//-----------------------------------------------------------------------------

void LanceCluster::Refresh()
{
  if (m_is_active)
  {
    if (!cluster_bomb.is_active) Explosion();
  } else {
    double angle = ActiveTeam().viseur.LitAngle();
    m_image.set_angle(angle);
  }

  m_draw_weapon = !m_is_active && (boucle_jeu.LitEtat() != jeuVER_A_JOUE);
}

//-----------------------------------------------------------------------------

void LanceCluster::Init()
{
#ifdef MSG_DBG
  COUT_DBG << "LanceCluster::Init()" << std::endl;
#endif

  m_name = _("ClusterBomb");
  cluster_bomb.Init();
  m_image = CL_Surface("clusterbomb", &graphisme.weapons);
  impact = CL_Surface("grenade_impact", &graphisme.weapons);
  icone = CL_Surface("clusterbomb_icon", &graphisme.weapons);
}

//-----------------------------------------------------------------------------

void LanceCluster::Reset()
{}

//-----------------------------------------------------------------------------

ClusterBombConfig& LanceCluster::cfg() 
{ return static_cast<ClusterBombConfig&>(*extra_params); }

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

ClusterBombConfig::ClusterBombConfig()
{
  tps_avt_explosion = 10;
  coeff_rebond = 75;
  nbr_fragments = 5;
}


void ClusterBombConfig::LoadXml(xmlpp::Element *elem)
{
  ExplosiveWeaponConfig::LoadXml(elem);
  LitDocXml::LitUint (elem, "tps_avt_explosion", tps_avt_explosion);
  LitDocXml::LitUint (elem, "coeff_rebond", coeff_rebond);
  LitDocXml::LitUint (elem, "nbr_fragments", nbr_fragments);
}

//-----------------------------------------------------------------------------
} // namespace Wormux
