/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Arme Sainte grenade : Comme la grenade mais en plus puissant :-)
 * Explose au bout de quelques secondes
 *****************************************************************************/

#include "grenade.h"
//-----------------------------------------------------------------------------
#include <sstream>
#include "../game/time.h"
#include "../team/teams_list.h"
#include "../interface/interface.h"
#include "../graphic/video.h"
#include "../tool/math_tools.h"
#include "../game/game_loop.h"
#include "../graphic/graphism.h"
#include "../map/camera.h"
#include "../weapon/weapon_tools.h"
#include "../interface/game_msg.h"
#include "../tool/i18n.h"
#include "../object/objects_list.h"
#include "../sound/jukebox.h"
//-----------------------------------------------------------------------------
namespace Wormux {
LanceGrenade lance_grenade;
//-----------------------------------------------------------------------------

#ifdef DEBUG

// #define MSG_DBG

#define COUT_DBG std::cout << "[Grenade] "

#endif

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

Grenade::Grenade() : ProjectileArme ("grenade")
{
  m_allow_negative_y = true;
  m_rebound_sound = "weapon/grenade_bounce";
  m_rebounding = true;
  touche_ver_objet = false;
}

//-----------------------------------------------------------------------------

void Grenade::Tire (int force)
{
  PrepareTir();

  // Fixe la position de dpart
  int x,y;
  ActiveTeam().LitArme().PosArmeXY (x, y);
  SetXY (x,y);

  NewSpeedXY(0.0,0.0);
  // Fixe la force de dpart
  double angle = ActiveTeam().viseur.LitAngle();
  NewStrength (force, angle);

#ifdef MSG_DBG
  COUT_DBG << "Grenade::Tire()" << std::endl;
#endif

  // Recupere le moment du dpart
  temps_debut_tir = Wormux::temps.Lit();
}

//-----------------------------------------------------------------------------

void Grenade::Init()
{
  image = CL_Sprite("grenade_sprite", &graphisme.weapons);
  SetSize (image.get_width(), image.get_height());

  SetMass (lance_grenade.cfg().masse);
  m_rebound_factor = double(lance_grenade.cfg().coeff_rebond)/100;

  // Fixe le rectangle de test
  int dx = image.get_width()/2-1;
  int dy = image.get_height()/2-1;
  SetTestRect (dx, dx, dy, dy);

#ifdef MSG_DBG
  COUT_DBG << "Grenade::Init()" << std::endl;
#endif
}

//-----------------------------------------------------------------------------

void Grenade::Refresh()
{
  if (!is_active) return;

#ifdef MSG_DBG
  COUT_DBG << "Grenade::Refresh()" << std::endl;
#endif

  if (IsGhost())
  {
    messages_jeu.Ajoute ("The grenade left the battlefield before exploding.");
    is_active = false;
    return;
  }

  //5 sec aprs avoir t tire, la grenade explose
  double tmp = Wormux::temps.Lit() - temps_debut_tir;
  if(tmp>1000 * lance_grenade.cfg().tps_avt_explosion) {
    is_active = false;
    return;
  }

  if (TestImpact()) { SignalCollision(); return; }

  // rotation de l'image de la grenade...
  double angle = CalculateMvtAngle() * 180/M_PI ;
  image.set_angle(angle);
}

//-----------------------------------------------------------------------------

void Grenade::Draw()
{
  if (!is_active) return;
  if (IsGhost())
  {
    messages_jeu.Ajoute ("The grenade left the battlefield before exploding");
    is_active = false;
    return;
  }

  image.draw(GetX(),GetY());
  int tmp = lance_grenade.cfg().tps_avt_explosion;
  tmp -= (int)((Wormux::temps.Lit() - temps_debut_tir) / 1000);
  std::ostringstream ss;
  ss << tmp;
  int txt_x = GetX() + GetWidth() / 2;
  int txt_y = GetY() - GetHeight();
  police_mix.EcritCentre (txt_x, txt_y, ss.str());
}

//-----------------------------------------------------------------------------

void Grenade::SignalCollision()
{   
  if (IsGhost())
  {
    messages_jeu.Ajoute ("The grenade left the battlefield before exploding");
    is_active = false ;
  }
}

//-----------------------------------------------------------------------------

void Grenade::Reset()
{
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

LanceGrenade::LanceGrenade() : Arme("grenade")
{
  m_is_active = false;
  extra_params = new GrenadeConfig();
}

//-----------------------------------------------------------------------------

bool LanceGrenade::Tire ()
{
  m_is_active = true;

  if (!UtiliseMunition()) return false;

  ActiveCharacter().previous_strength = m_strength;

  // Initialise la grenade
  grenade.Tire (m_strength);
  camera.ChangeObjSuivi (&grenade, true, false);
  lst_objets.AjouteObjet (&grenade, true);
  m_strength = 0;

  jukebox.PlayProfile(ActiveTeam().LitProfileSon(), "fire");

#ifdef MSG_DBG
  COUT_DBG << "LanceGrenade::Tire()" << std::endl;
#endif

  return true;
}

//-----------------------------------------------------------------------------

void LanceGrenade::Explosion()
{
  m_is_active = false;

#ifdef MSG_DBG
  COUT_DBG << "LanceGrenade::Explosion()" << std::endl;
#endif
  
  lst_objets.RetireObjet (&grenade);

  // On fait un trou ?
  if (grenade.IsGhost()) return;

  // Applique les degats et le souffle aux vers
  CL_Point pos = grenade.GetCenter();
  AppliqueExplosion (pos, pos, impact, cfg(), NULL);
}

//-----------------------------------------------------------------------------

void LanceGrenade::Refresh()
{
  if (m_is_active)
  {
    if (!grenade.is_active) Explosion();
  } 
}

//-----------------------------------------------------------------------------

void LanceGrenade::Init()
{
  m_name = _("Grenade");
  grenade.Init();
  m_image = CL_Surface("grenade", &graphisme.weapons);
  impact = CL_Surface("grenade_impact", &graphisme.weapons);
  icone = CL_Surface("grenade_icon", &graphisme.weapons);
}

//-----------------------------------------------------------------------------

void LanceGrenade::Reset()
{}

//-----------------------------------------------------------------------------

GrenadeConfig& LanceGrenade::cfg() 
{ return static_cast<GrenadeConfig&>(*extra_params); }

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

GrenadeConfig::GrenadeConfig()
{
  tps_avt_explosion = 10;
  coeff_rebond = 75;
}

void GrenadeConfig::LoadXml(xmlpp::Element *elem)
{
  ExplosiveWeaponConfig::LoadXml(elem);
  LitDocXml::LitUint (elem, "tps_avt_explosion", tps_avt_explosion);
  LitDocXml::LitUint (elem, "coeff_rebond", coeff_rebond);
}

//-----------------------------------------------------------------------------

} // namespace Wormux
