/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Arme pistolet : la balle part tout droit dans la direction donne par
 * le viseur. Si la balle ne touche pas un ver, elle va faire un trou dans
 * le terrain. La balle peut galement toucher les objets du plateau du jeu.
 *****************************************************************************/

#include "../weapon/gun.h"
//-----------------------------------------------------------------------------
#include <sstream>
#include "../game/game_loop.h"
#include "../map/map.h"
#include "../game/time.h"
#include "../object/objects_list.h"
#include "../team/teams_list.h"
#include "../graphic/graphism.h"
#include "../tool/i18n.h"
#include "../interface/game_msg.h"
#include "../interface/game_msg.h"
#include "../sound/jukebox.h"
#include "../weapon/gun.h"
using namespace std;
//-----------------------------------------------------------------------------
namespace Wormux {
Pistolet pistolet;
//-----------------------------------------------------------------------------

const uint VITESSE_CAPTURE_POS_BALLE = 10;
const uint VITESSE_BALLE = 500;
const double SOUFFLE_BALLE = 50;

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

BallePistolet::BallePistolet() : ProjectileArme("balle_pistolet")
{ 
  touche_ver_objet = true; 
}

//-----------------------------------------------------------------------------

void BallePistolet::Reset()
{}

//-----------------------------------------------------------------------------

void BallePistolet::Init()
{
  image = CL_Sprite("gun_bullet", &graphisme.weapons);
  SetSize (image.get_width(), image.get_height());
  SetMass (0);
}

//-----------------------------------------------------------------------------

void BallePistolet::Tire()
{
  PrepareTir();

  // Place la balle
  int x,y;
  ActiveTeam().LitArme().RotationPointXY (x, y);
  x -= image.get_width()/2;
  y -= image.get_height()/2;
  SetXY (x,y);

  // Donne une force initiale
  angle = ActiveTeam().viseur.LitAngle();
  NewSpeedXY(0.0,0.0);
  NewStrength (VITESSE_BALLE, angle);
}

//-----------------------------------------------------------------------------

void BallePistolet::SignalCollision()
{ 
  if ((dernier_ver_touche == NULL) && (dernier_obj_touche == NULL))
  {
    messages_jeu.Ajoute (_("Your shot has missed!"));
  }
  is_active = false; 
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

Pistolet::Pistolet() : Arme("gun")
{
  nb_bullets = MUNITION_ILLIMITEES;
  extra_params = new WeaponConfig();
}

//-----------------------------------------------------------------------------

void Pistolet::Reset()
{
  balle.Reset();
  m_is_active = false;
}

//-----------------------------------------------------------------------------

void Pistolet::Init()
{
  m_name = _("Gun");
  balle.Init();
  impact = CL_Surface("gun_impact", &graphisme.weapons);  
  m_image = CL_Surface("gun", &graphisme.weapons);
  icone = CL_Surface("gun_icon", &graphisme.weapons);
}

//-----------------------------------------------------------------------------

void Pistolet::Draw ()
{
  Arme::Draw();

  // Dessine le trac de la trajectoire
  if (m_is_active)
  {
    bool noir=true;
    for (std::vector<CL_Point>::const_iterator it=lst_points.begin();
	 it != lst_points.end();
	 ++it)
    {
      if (noir)
	CL_Display::draw_rect (CL_Rect(it -> x, it -> y, it -> x+1, it -> y+1),
			       CL_Color::black);
      else
	CL_Display::draw_rect (CL_Rect(it -> x, it -> y, it -> x+1, it -> y+1),
			       CL_Color::white);
      noir = !noir;
    }
  }
}

//-----------------------------------------------------------------------------

bool Pistolet::Tire()
{
  if (!UtiliseMunition()) return false;

  // Initialise la balle
  balle.Tire();
  m_is_active = true;

  // Temps de capture
  temps_capture = temps.Lit()+VITESSE_CAPTURE_POS_BALLE;

  lst_points.clear();
  lst_objets.AjouteObjet (&balle, true);

  // Cache le curseur
    jukebox.Play("weapon/gun");
  
  return m_is_active;
}

//-----------------------------------------------------------------------------

void Pistolet::Refresh()
{
  m_image.set_scale(ActiveCharacter().GetDirection(), 1);

  if (balle.is_active) {
    // Une balle est en l'air : on capture sa position ?
    if (temps_capture < temps.Lit()) 
    {
      temps_capture = temps.Lit()+VITESSE_CAPTURE_POS_BALLE;
      CL_Point pos_balle = balle.GetPos();
      pos_balle.x += balle.GetWidth()/2;
      pos_balle.y += balle.GetHeight()/2;
      lst_points.push_back (pos_balle);
    }
  } else {
    if (!m_is_active) return;
    m_is_active = false;

    if (!balle.IsGhost())
    {
      // Si la balle a touch un ver, lui inflige des dgats
      Character* ver = balle.LitDernierVerTouche();
      PhysicalObj* obj = balle.LitDernierObjTouche();
      if (ver) obj = ver;
      if (ver) ver -> SetEnergyDelta (-cfg().degats);
      if (obj) 
      {
	obj -> AddStrength (SOUFFLE_BALLE, balle.LitAngle());
      }

      // Creuse le monde
      if (!obj)
      {
	monde.Creuse (balle.GetX() - impact.get_width()/2,
		      balle.GetY() - impact.get_height()/2,
		      impact);
      }
    }

    lst_objets.RetireObjet (&balle);
  }
}

//-----------------------------------------------------------------------------

WeaponConfig& Pistolet::cfg()
{ return static_cast<WeaponConfig&>(*extra_params); }

//-----------------------------------------------------------------------------
} // namespace Wormux
