/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Classes virtuelles permettant de dfinir une arme et un projectile. Les
 * armes ont un nom, une image, un tat actif/inactif et une icne (affiche
 * dans l'interface). Les projectiles sont des objets physiques qui ont un
 * comportement spcial lorsqu'ils entrent en collision ou qu'ils sortent du
 * terrain.
 *****************************************************************************/

#include "../weapon/weapon.h"
//-----------------------------------------------------------------------------
#include "../team/macro.h"
#include "../tool/i18n.h"
#include "../tool/geometry_tools.h"
#include "../tool/math_tools.h"
#include "../tool/xml_document.h"
#include "../map/camera.h"
#include "../object/objects_list.h"
#include "../sound/jukebox.h"
#include "../interface/cursor.h"
#include "../gui/progress_bar.h"
#include "weapon_tools.h"
#include "../game/time.h"
#include "../game/game_loop.h"
#include <iostream>
//-----------------------------------------------------------------------------

#ifdef DEBUG
#define COUT_DBG std::cout<<"[Weapon "<<m_name<<"] "
//# define DEBUG_CADRE_TEST
//#define DEBUG_MSG_COLLISION
#endif

//-----------------------------------------------------------------------------
const int MUNITION_ILLIMITEES = -1;
const uint MAX_TIME_LOADING = 2000;
//-----------------------------------------------------------------------------
extern BarreProg weapon_strength_bar;
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

ProjectileArme::ProjectileArme (const std::string &name)
  : PhysicalObj (name, 0.0)
{
  dernier_ver_touche = NULL;
  dernier_obj_touche = NULL;
  touche_ver_objet = false;
}

//-----------------------------------------------------------------------------

void ProjectileArme::PrepareTir()
{
  SetSize (image.get_width(), image.get_height());
  Ready();
  camera.ChangeObjSuivi (this, true, false);
  is_active = true;
}

//-----------------------------------------------------------------------------

bool ProjectileArme::TestImpact()
{
  if (IsReady()) 
  {
#ifdef DEBUG_MSG_COLLISION
    COUT_DBG << "Impact because was ready." << std::endl;
#endif
    return true;
  }
  return CollisionTest (0,0);
}

//-----------------------------------------------------------------------------

bool ProjectileArme::CollisionTest(int dx, int dy)
{
  dernier_ver_touche = NULL;
  dernier_obj_touche = NULL;

  if (!IsInVacuum (dx, dy)) return true;

  if (!touche_ver_objet) return false;

  CL_Rect test = GetTestRect();
  MoveRect (test, dx, dy);

  POUR_TOUS_VERS_VIVANTS(equipe,ver)
  if (&(*ver) != &ActiveCharacter())
  {
    if (RectTouche(ver -> GetTestRect(), test)) 
    {
      dernier_ver_touche = &(*ver);
#ifdef DEBUG_MSG_COLLISION
      COUT_DBG << "On a touch le ver : " << ver -> m_name << std::endl;
#endif
      return true;
    }
  }

  POUR_CHAQUE_OBJET(objet)
  if (objet -> ptr != this)
  {
    if (RectTouche(objet -> ptr -> GetTestRect(), test)) 
    {
      dernier_obj_touche = objet -> ptr;
#ifdef DEBUG_MSG_COLLISION
      COUT_DBG << "On a touch un objet : " 
                << objet -> ptr -> m_name << std::endl;      
#endif
      return true;
    }
  }

  return false;
}

//-----------------------------------------------------------------------------

void ProjectileArme::Refresh()
{
  if (!is_active) return;

  if (TestImpact()) { SignalCollision(); return; }
}

//-----------------------------------------------------------------------------

void ProjectileArme::Draw()
{
  if (!is_active) return;
  image.draw (GetX(), GetY());
#if defined(DEBUG_CADRE_TEST)
  CL_Display::draw_rect (GetTestRect(), CL_Color::red);
#endif
}

//-----------------------------------------------------------------------------

void ProjectileArme::SignalGhostState (bool) { SignalCollision(); }

//-----------------------------------------------------------------------------

void ProjectileArme::SignalFallEnding() { SignalCollision(); }

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

Arme::Arme (const std::string &id)
{
  m_id = id;

  m_is_active = false;
  m_draw_weapon = true;

  category = 1;

  nb_bullets = MUNITION_ILLIMITEES;

  m_strength = 0;
  m_first_time_loading = 0;

  min_angle = max_angle = 0;

  position.origin = weapon_origin_HAND;
  position.dx = 0;
  position.dy = 0;

  m_nb_replay = 0;
  m_can_change_weapon = false;

  extra_params = NULL;
}

//-----------------------------------------------------------------------------
/* ARGHH  ca peut pas marcher  cause de l'internationalisation !!
Arme::Init ()
{
  CL_ResourceManager *res=graphisme.LitRes();
  image = CL_Surface("weapon/"+nom, res);
  icone = CL_Surface("ico_weapon/"+nom, res);
}
*/
//-----------------------------------------------------------------------------

void Arme::Active() 
{
  // is there a crosshair ?
  if (min_angle != max_angle) {
    ActiveTeam().viseur.actif = true; 
  }

  if (max_strength == 0) return ;
  
  // prepare the strength bar
  weapon_strength_bar.InitVal (0, 0, max_strength);

  // init stamp on the stength_bar
  uint val = ActiveCharacter().previous_strength;
  weapon_strength_bar.Reset_Marqueur();
  if (0 < val && val < max_strength) weapon_strength_bar.AjouteMarqueur (val, CL_Color::red);
}

//-----------------------------------------------------------------------------

void Arme::Desactive()
{
  ActiveTeam().viseur.actif = false;
  m_nb_playing = 0;
}

//-----------------------------------------------------------------------------

void Arme::Manage()
{
  // are we still playing ?
  if (  m_nb_playing > m_nb_replay && m_can_change_weapon == false)
  {
    Desactive();
    boucle_jeu.ChangeEtat (jeuVER_A_JOUE); // No !
  }

  Refresh();
}

//-----------------------------------------------------------------------------

bool Arme::CanChangeWeapon() const
{
  if (m_can_change_weapon == true || m_nb_playing == 0) return true ;
  return false;
}

//-----------------------------------------------------------------------------

bool Arme::Shoot()
{
  m_nb_playing++ ;

  return Tire() ;
}

//-----------------------------------------------------------------------------

// Calcule la position de l'image de l'arme
void Arme::PosArmeXY (int &x, int &y) const
{
  int hand_x, hand_y;
  ActiveCharacter().GetHandPosition(hand_x, hand_y);
  y = hand_y +position.dy;
  if (ActiveCharacter().GetDirection() == 1)
    x = hand_x +position.dx;
  else {
    x = hand_x -position.dx;
  }

  if(min_angle!=max_angle && ActiveCharacter().GetDirection()==-1)
    x -= m_image.get_width();
}

//-----------------------------------------------------------------------------

// Return the absolute rotation point of the weapon
void Arme::RotationPointXY (int &x, int &y) const
{
  PosArmeXY(x,y);
  x += m_image.get_width()/2;
  y += m_image.get_height()/2;
}


//-----------------------------------------------------------------------------

bool Arme::MunitionSuffisante() const
{
  int munition = ActiveTeam().ReadNbBullets();
  return ((munition == MUNITION_ILLIMITEES) || (0 < munition));
}

//-----------------------------------------------------------------------------

bool Arme::UtiliseMunition()
{
  // On a assez de munition ?
  if (!MunitionSuffisante()) return false;

  // Utilise une munition
  int *munition = &ActiveTeam().AccessNbBullets();
  if (*munition != MUNITION_ILLIMITEES) (*munition)--;
  return true;
}

//-----------------------------------------------------------------------------

const std::string& Arme::GetName() const 
{ 
  assert (!m_name.empty());
  return m_name;
}

//-----------------------------------------------------------------------------

void Arme::UpdateStrength()
{
  if (max_strength == 0 || m_first_time_loading == 0) return ;
  
  uint time = Wormux::temps.Lit() - m_first_time_loading;
  uint val = (max_strength * time) / MAX_TIME_LOADING;
  
  m_strength = BorneLong (val, 0, max_strength);
  UpdateDrawStrengthBar(m_strength);
}


//-----------------------------------------------------------------------------

bool Arme::IsReady() const
{
  return MunitionSuffisante() ;
}
//-----------------------------------------------------------------------------

void Arme::InitLoading()
{
  // no loading for weapon with max_strength = 0
  if (max_strength == 0) return ;

  jukebox.Play("weapon/load");
  curseur_ver.Cache();

  //is_loading = true;
  m_first_time_loading = Wormux::temps.Lit();
  
  m_strength = 0;
}
//-----------------------------------------------------------------------------

void Arme::StopLoading()
{
  m_first_time_loading = 0 ;
  jukebox.Stop("weapon/load");
}
//-----------------------------------------------------------------------------

void Arme::Draw()
{
  if (!m_draw_weapon) return;
  if (m_is_active || (boucle_jeu.LitEtat() == jeuVER_A_JOUE)) return;

  if(ActiveCharacter().IsGhost()
  || ActiveCharacter().IsDrowned()
  || ActiveCharacter().IsDead())
    return;

  // show strength_bar if necessay
  if (max_strength != 0 && IsReady())
      UpdateDrawStrengthBar(m_strength);

  // rotate weapon if necessary
  if (min_angle != max_angle) {
    double angle = ActiveTeam().viseur.LitAngle();
    m_image.set_rotation_hotspot (origin_center);
    m_image.set_angle (angle * 180/M_PI);
    m_image.set_scale(1, ActiveCharacter().GetDirection());
  } else {
    m_image.set_scale(ActiveCharacter().GetDirection(), 1);
  }

  // Calculate position of the image
  int x,y;
  switch (position.origin) 
  {
  case weapon_origin_OVER:
    x = ActiveCharacter().GetCenterX()-m_image.get_width()/2+position.dx;
    y = ActiveCharacter().GetY()-m_image.get_height()+position.dy;
    if(ActiveCharacter().GetDirection() == -1)
      x += m_image.get_width();
    break;
  case weapon_origin_HAND:
    PosArmeXY (x, y);
    if(min_angle!=max_angle && ActiveCharacter().GetDirection()==-1)
      y += m_image.get_height();
    break;
  }
  m_image.draw (x,y);

#if defined(DEBUG_CADRE_TEST)
  CL_Display::draw_rect (CL_Rect(x,y,
				 x+m_image.get_width(), 
				 y+m_image.get_height()),
			 CL_Color::red);
#endif
}

//-----------------------------------------------------------------------------

bool Arme::LoadXml(xmlpp::Element * weapon) 
{
  xmlpp::Element *elem = LitDocXml::AccesBalise (weapon, m_id);
  if (elem == NULL)
  {
      std::cout << Format(_("No element <%s> found in the xml config file !"),
                          m_id.c_str())
                << std::endl;
    return false;
  }

  xmlpp::Element *pos_elem = LitDocXml::AccesBalise (elem, "position");
  if (pos_elem != NULL) {
    // E.g. <position origin="hand" x="-1" y="0" />
    std::string origin;
    LitDocXml::LitAttrInt (pos_elem, "x", position.dx);
    LitDocXml::LitAttrInt (pos_elem, "y", position.dy);
    LitDocXml::LitAttrString (pos_elem, "origin", origin);
    if (origin == "over")
      position.origin = weapon_origin_OVER;
    else
      position.origin = weapon_origin_HAND;
  }

  // What is the weapon's category ? (used for key-bindings)
  LitDocXml::LitUint (elem, "category", category);       
  
  LitDocXml::LitInt (elem, "bullets", nb_bullets);       

  // max strength
  // if max_strength = 0, no strength_bar !
  LitDocXml::LitUint (elem, "max_strength", max_strength);

  // replay after (in the same turn) ?
  LitDocXml::LitUint (elem, "nb_replay", m_nb_replay);      

  // change weapon after ? (for the ninja cord = true)
  LitDocXml::LitBool (elem, "change_weapon", m_can_change_weapon);
      
  // angle of weapon when drawing
  // if (min_angle == max_angle) no cross_hair !
  // between -90 to 90 degrees
  LitDocXml::LitInt (elem, "min_angle", min_angle);
  LitDocXml::LitInt (elem, "max_angle", max_angle);

  // Load extra parameters if existing
  if (extra_params != NULL) extra_params->LoadXml(elem);

  return true;
}

//-----------------------------------------------------------------------------

bool Arme::IsActive() const { return m_is_active; }
const uint Arme::ReadStrength() const { return m_strength; }
bool Arme::IsLoading() const { return m_first_time_loading; }


//-----------------------------------------------------------------------------
