/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Utilitaires pour les armes : applique une explosion en un point.
 *****************************************************************************/

#include "../weapon/weapon_tools.h"
//-----------------------------------------------------------------------------
#include "../team/macro.h"
#include "../map/map.h"
#include "../tool/geometry_tools.h"
#include "../sound/jukebox.h"
#include "../tool/math_tools.h"
#include "../object/objects_list.h"
#include "../weapon/weapons_list.h"
#include "../graphic/video.h"
#include "../interface/interface.h"
#include "../map/camera.h"
//-----------------------------------------------------------------------------

#ifdef DEBUG
//#  define DEBUG_EXPLOSION
#  define COUT_DBG cout << "[Explosion] "
#endif


//-----------------------------------------------------------------------------
extern BarreProg weapon_strength_bar;
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

void AppliqueExplosion (const CL_Point &explosion, 
			const CL_Point &trou, 
			CL_Surface &impact,
			const ExplosiveWeaponConfig &config,
			PhysicalObj *obj_exclu,
			const std::string& son
			)
{
  // Fait un trou dans le terrain
  monde.Creuse (trou.x - impact.get_width()/2, trou.y-impact.get_height()/2,
		impact);

  // Joue un son
  jukebox.Play (son);

  // Applique les degats aux vers
  // Ne se soucie pas de la mort du ver actif
  POUR_TOUS_VERS(equipe,ver)
  { 
    // C'est l'objet qu'on ne veut pas traiter ?
    if ((obj_exclu != NULL) && (&(*ver) == obj_exclu)) continue;

    double norme, angle;
    norme = Distance (explosion, ver -> GetCenter());
#ifdef DEBUG_EXPLOSION
    COUT_DBG << " ver " << ver -> nom << " : norme=" << norme;
#endif

    // Appliques les dgats au ver
    if (norme <= config.dst_explosion)
    {
      uint perte = (uint)(norme*config.degats/config.dst_explosion);
      perte = config.degats-perte;
#ifdef DEBUG_EXPLOSION
      COUT_DBG << "Perte nergie=" << perte << endl;
#endif
      ver -> SetEnergyDelta (-perte);
    }

    // Applique le souffle au ver
    if (norme <= config.dst_souffle)
    {
      double force = norme;
      if (!EgalZero(norme)) {
	force *= config.force_souffle / config.dst_souffle;
	force  = config.force_souffle - force;
	angle  = CalculeAngle (explosion, ver -> GetCenter());
      } else {
	force = config.force_souffle;
	angle = -M_PI/2;
      }
#ifdef DEBUG_EXPLOSION
      cout << ", force=" << force << endl;
#endif
      ver -> AddStrength (force, angle);
      ver -> PhysicalEngine();
    } else {
#ifdef DEBUG_EXPLOSION
      cout << " -> trop loin." << endl;
#endif
    }

    // Mise  jour de l'tat du ver
    ver -> Refresh();
#ifdef DEBUG_EXPLOSION
    COUT_DBG << ver -> nom << " fantome " << ver -> IsGhost()
	     << " mort " << ver -> IsDead() 
	     << " actif " << ver -> EstActif() << endl;
#endif
  }

  // Applique le souffle aux objets physiques
  POUR_CHAQUE_OBJET(obj) if (obj -> ptr -> GetObjectType() == objCLASSIC)
  { 
    // C'est l'objet qu'on ne veut pas traiter ?
    if ((obj_exclu != NULL) && (obj -> ptr == obj_exclu)) continue;

    double norme, angle;
    norme = Distance (explosion, obj -> ptr -> GetCenter());
    if (norme <= config.dst_souffle)
    {
      if (!EgalZero(norme)) {
	norme *= config.force_souffle / config.dst_souffle;
	norme  = config.force_souffle - norme;
	angle  = CalculeAngle (explosion, obj -> ptr -> GetCenter());
      } else {
	norme = config.force_souffle;
	angle = -M_PI/2;
      }
      obj -> ptr -> AddStrength (norme, angle);
      obj -> ptr -> PhysicalEngine();
    }
  }
}

//-----------------------------------------------------------------------------

void UpdateDrawStrengthBar(uint strength)
{
  // Update strength
  weapon_strength_bar.Actu (strength);

  // Position on the screen 
  uint barre_x = (video.GetWidth()-weapon_strength_bar.GetWidth())/2;
  uint barre_y = video.GetHeight();

  if (interface.EstAffiche()) barre_y -= interface.GetHeight();
  barre_y -= weapon_strength_bar.GetHeight() +10;

  // Drawing on the screen
  weapon_strength_bar.DrawXY (camera.GetX() +barre_x, camera.GetY() +barre_y);
}


//-----------------------------------------------------------------------------
