/******************************************************************************
 *  Wormux, a free clone of the game Worms from Team17.
 *  Copyright (C) 2001-2004 Lawrence Azzoug.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 ******************************************************************************
 * Refresh des armes.
 *****************************************************************************/

#include "../weapon/weapons_list.h"
//-----------------------------------------------------------------------------
#include "../team/macro.h"
#include "../game/game_loop.h"
#include "../object/objects_list.h"
#include "../map/camera.h"
#include "../interface/interface.h"
#include "all.h"
using namespace Wormux;
//-----------------------------------------------------------------------------
RefreshArmes gst_armes;
//-----------------------------------------------------------------------------
BarreProg weapon_strength_bar;

const uint BAR_WIDTH = 400;
const uint BAR_HEIGHT = 10;
//-----------------------------------------------------------------------------

RefreshArmes::RefreshArmes()
{
}

//-----------------------------------------------------------------------------

void RefreshArmes::InitAndAddToList(Arme* arme, uint num_sort)
{
  arme->Init();

  // insert the pointer
  m_weapons_list.push_back(arme);

  m_weapons_map.insert(keybind(num_sort, arme));

  interface.menu_armes.NewItem(arme,num_sort);
}

//-----------------------------------------------------------------------------

void RefreshArmes::Init()
{
  InitAndAddToList(&bazooka, 1);
  InitAndAddToList(&auto_bazooka, 1);
  InitAndAddToList(&lance_grenade, 1);
  InitAndAddToList(&lance_cluster, 1);
  InitAndAddToList(&pistolet, 2);
  InitAndAddToList(&baseball, 2);  
  InitAndAddToList(&dynamite,3);
  InitAndAddToList(&mine,3);
  InitAndAddToList(&aerienne,4);
  InitAndAddToList(&teleportation,5);
  InitAndAddToList(&suicide,5);
  InitAndAddToList(&skipturn,5);
  
  // strength bar initialisation  
  weapon_strength_bar.InitPos (0, 0, BAR_WIDTH, BAR_HEIGHT);
  weapon_strength_bar.InitVal (0, 0, 100);
}

//-----------------------------------------------------------------------------

void RefreshArmes::Refresh ()
{
  ActiveTeam().AccesArme().Manage();
}

//-----------------------------------------------------------------------------

std::list<Arme *>& RefreshArmes::GetList()
{
  return m_weapons_list;
}

//-----------------------------------------------------------------------------

Arme * RefreshArmes::GetWeaponBySort(uint sort)
{
  uint nb_weapons = m_weapons_map.count(sort);
  if (nb_weapons == 0) return NULL;
	
  // One or many weapons on this key
  std::pair<std::multimap<uint, Arme*>::iterator, 
    std::multimap<uint, Arme*>::iterator> p = m_weapons_map.equal_range(sort);
  
  std::multimap<uint, Arme*>::iterator it = p.first;
  std::multimap<uint, Arme*>::iterator end = p.second;
  
  // we turn between the differents possibility
  if (nb_weapons > 1)
    {
      while ( (*it).second != &(ActiveTeam().LitArme()) && it != end) it++ ;
      
      // the previous selected weapon has been founded
      if (it != end) it++;

      if (it == end) it = p.first;
    } 
  
  // then we selected the "next" one
  return ( (*it).second );
}
//-----------------------------------------------------------------------------


